#ifndef CARD_READER_H
#define CARD_READER_H


#include <winscard.h>
#include <dshow.h>
#include "BonBaseClass.h"


// J[h[_[NX
class __declspec(novtable) CCardReader : public CBonBaseClass
{
public:
	enum ReaderType {
		READER_NONE,
		READER_SCARD,
		READER_SCARD_DYNAMIC,
		READER_HDUS,
		READER_LAST=READER_HDUS
	};
private:
	ReaderType m_ReaderType;
public:
	CCardReader();
	virtual ~CCardReader();
	virtual bool Open(LPCTSTR pszReader=NULL)=0;
	virtual void Close()=0;
	virtual LPCTSTR GetReaderName() const=0;
	virtual int NumReaders() const { return 1; }
	virtual LPCTSTR EnumReader(int Index) const;
	ReaderType GetReaderType() const { return m_ReaderType; }
	virtual bool Transmit(const void *pSendData,DWORD SendSize,void *pRecvData,DWORD *pRecvSize)=0;
	static CCardReader *CreateCardReader(ReaderType Type);
};

// X}[gJ[h[_[(X^eBbNN)
class CSCardReader : public CCardReader
{
	SCARDCONTEXT m_ScardContext;
	SCARDHANDLE m_hBcasCard;
	bool m_bIsEstablish;
	LPTSTR m_pReaderList;
	int m_NumReaders;
	LPTSTR m_pszReaderName;
public:
	CSCardReader();
	~CSCardReader();
	bool Open(LPCTSTR pszReader);
	void Close();
	LPCTSTR GetReaderName() const;
	int NumReaders() const;
	LPCTSTR EnumReader(int Index) const;
	bool Transmit(const void *pSendData,DWORD SendSize,void *pRecvData,DWORD *pRecvSize);
};

// X}[gJ[h[_[(_Ci~bNN)
// 킴킴X^eBbNNƕĂ̂́ABonCasLinko[Wɂ
// X^eBbNNłȂΗpłȂ
class CDynamicSCardReader : public CCardReader
{
	HMODULE m_hLib;
	SCARDCONTEXT m_ScardContext;
	SCARDHANDLE m_hBcasCard;
	LPTSTR m_pReaderList;
	int m_NumReaders;
	LPTSTR m_pszReaderName;
	typedef LONG (WINAPI *SCardTransmitFunc)(SCARDHANDLE,LPCSCARD_IO_REQUEST,LPCBYTE,
											 DWORD,LPSCARD_IO_REQUEST,LPBYTE,LPDWORD);
	SCardTransmitFunc m_pSCardTransmit;
	bool Load(LPCTSTR pszFileName);
public:
	CDynamicSCardReader();
	~CDynamicSCardReader();
	bool Open(LPCTSTR pszReader);
	void Close();
	LPCTSTR GetReaderName() const;
	int NumReaders() const;
	LPCTSTR EnumReader(int Index) const;
	bool Transmit(const void *pSendData,DWORD SendSize,void *pRecvData,DWORD *pRecvSize);
};

// HDUSJ[h[_[
class CHdusCardReader : public CCardReader
{
	IBaseFilter *m_pTuner;
	bool m_bSent;
	IBaseFilter *FindDevice(REFCLSID category,BSTR varFriendlyName);
	HRESULT Send(const void *pSendData,DWORD SendSize);
	HRESULT Receive(void *pRecvData,DWORD *pRecvSize);
public:
	CHdusCardReader();
	~CHdusCardReader();
	bool Open(LPCTSTR pszReader);
	void Close();
	LPCTSTR GetReaderName() const;
	bool Transmit(const void *pSendData,DWORD SendSize,void *pRecvData,DWORD *pRecvSize);
};


#endif
