#include "stdafx.h"
#include "RecTest.h"
#include "CoreEngine.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif




CCoreEngine::CCoreEngine()
{
	m_szDriverFileName[0]='\0';
	m_hDriverLib=NULL;
	m_DriverType=DRIVER_UNKNOWN;
	m_fDescramble=true;
	m_CardReaderType=CCardReader::READER_SCARD;
	m_pDtvEngineHandler=NULL;
	m_EventID=0;
	m_ErrorPacketCount=0;
	m_ContinuityErrorPacketCount=0;
	m_ScramblePacketCount=0;
	m_SignalLevel=0.0;
	m_BitRate=0;
	m_PacketBufferUsedCount=0;
	m_StreamRemain=0;
	/*
	m_pEpgDataInfo=NULL;
	m_pEpgDataInfoNext=NULL;
	*/
}


CCoreEngine::~CCoreEngine()
{
	Close();
	/*
	delete m_pEpgDataInfo;
	delete m_pEpgDataInfoNext;
	*/
}


void CCoreEngine::Close()
{
	//m_DtvEngine.ReleaseSrcFilter();
	m_DtvEngine.CloseEngine();
	if (m_hDriverLib) {
		::FreeLibrary(m_hDriverLib);
		m_hDriverLib=NULL;
	}
}


bool CCoreEngine::BuildDtvEngine(CDtvEngineHandler *pDtvEngineHandler)
{
	if (!m_DtvEngine.BuildEngine(pDtvEngineHandler,
				m_fDescramble?m_CardReaderType!=CCardReader::READER_NONE:false)) {
		return false;
	}
	m_pDtvEngineHandler=pDtvEngineHandler;
	return true;
}


bool CCoreEngine::SetDriverFileName(LPCTSTR pszFileName)
{
	if (::lstrlen(pszFileName)>=MAX_PATH)
		return false;
	::lstrcpy(m_szDriverFileName,pszFileName);
	return true;
}


bool CCoreEngine::LoadDriver()
{
	UnloadDriver();
	m_hDriverLib=::LoadLibrary(m_szDriverFileName);
	return m_hDriverLib!=NULL;
}


bool CCoreEngine::UnloadDriver()
{
	if (m_hDriverLib) {
		CloseDriver();
		::FreeLibrary(m_hDriverLib);
		m_hDriverLib=NULL;
	}
	return true;
}


bool CCoreEngine::OpenDriver()
{
	if (m_hDriverLib==NULL)
		return false;
	m_DtvEngine.ReleaseSrcFilter();
	if (!m_DtvEngine.OpenSrcFilter_BonDriver(m_hDriverLib)) {
		SetError(m_DtvEngine.GetLastErrorException());
		return false;
	}
	LPCTSTR pszName=m_DtvEngine.m_BonSrcDecoder.GetTunerName();
	m_DriverType=DRIVER_UNKNOWN;
	if (pszName!=NULL) {
		if (::lstrcmpi(pszName,TEXT("HDUS"))==0)
			m_DriverType=DRIVER_HDUS;
		else if (::_tcsncmp(pszName,TEXT("UDP/"),4)==0)
			m_DriverType=DRIVER_UDP;
	}
	return true;
}


bool CCoreEngine::CloseDriver()
{
	return m_DtvEngine.ReleaseSrcFilter();
}


bool CCoreEngine::IsDriverOpen() const
{
	return m_DtvEngine.IsSrcFilterOpen();
}


bool CCoreEngine::OpenBcasCard()
{
	if (m_fDescramble) {
		if (!m_DtvEngine.OpenBcasCard(m_CardReaderType)) {
			SetError(m_DtvEngine.GetLastErrorException());
			return false;
		}
	}
	return true;
}


bool CCoreEngine::IsBcasCardOpen() const
{
	return m_DtvEngine.m_TsDescrambler.IsBcasCardOpen();
}


bool CCoreEngine::IsBuildComplete() const
{
	return m_DtvEngine.IsBuildComplete();
}


bool CCoreEngine::SetDescramble(bool fDescramble)
{
	if (m_fDescramble!=fDescramble) {
		if (m_DtvEngine.IsEngineBuild()) {
			if (!m_DtvEngine.SetDescramble(fDescramble)) {
				SetError(m_DtvEngine.GetLastErrorException());
				return false;
			}
		}
		m_fDescramble=fDescramble;
	}
	return true;
}


bool CCoreEngine::SetCardReaderType(CCardReader::ReaderType Type)
{
	if (Type<CCardReader::READER_NONE || Type>CCardReader::READER_LAST)
		return false;
	if (m_CardReaderType!=Type) {
		if (m_DtvEngine.IsEngineBuild()) {
			if (!SetDescramble(Type!=CCardReader::READER_NONE))
				return false;
			if (!m_DtvEngine.OpenBcasCard(Type)) {
				SetError(m_DtvEngine.GetLastErrorException());
				return false;
			}
		}
		m_CardReaderType=Type;
	}
	return true;
}


DWORD CCoreEngine::UpdateAsyncStatus()
{
	DWORD Updated=0;

	WORD EventID=m_DtvEngine.GetEventID();
	if (EventID!=m_EventID) {
		m_EventID=EventID;
		Updated|=STATUS_EVENTID;
		TRACE(TEXT("EventID = %d\n"),EventID);
	}
	return Updated;
}


DWORD CCoreEngine::UpdateStatistics()
{
	DWORD Updated=0;

	DWORD ErrorCount=m_DtvEngine.m_TsPacketParser.GetErrorPacketCount();
	if (ErrorCount!=m_ErrorPacketCount) {
		m_ErrorPacketCount=ErrorCount;
		Updated|=STATISTIC_ERRORPACKETCOUNT;
	}
	DWORD ContinuityErrorCount=m_DtvEngine.m_TsPacketParser.GetContinuityErrorPacketCount();
	if (ContinuityErrorCount!=m_ContinuityErrorPacketCount) {
		m_ContinuityErrorPacketCount=ContinuityErrorCount;
		Updated|=STATISTIC_CONTINUITYERRORPACKETCOUNT;
	}
	DWORD ScrambleCount=m_DtvEngine.m_TsDescrambler.GetScramblePacketCount();
	if (ScrambleCount!=m_ScramblePacketCount) {
		m_ScramblePacketCount=ScrambleCount;
		Updated|=STATISTIC_SCRAMBLEPACKETCOUNT;
	}
	float SignalLevel;
	DWORD BitRate;
	DWORD StreamRemain;
	SignalLevel=m_DtvEngine.m_BonSrcDecoder.GetSignalLevel();
	BitRate=m_DtvEngine.m_BonSrcDecoder.GetBitRate();
	StreamRemain=m_DtvEngine.m_BonSrcDecoder.GetStreamRemain();
	if (SignalLevel!=m_SignalLevel) {
		m_SignalLevel=SignalLevel;
		Updated|=STATISTIC_SIGNALLEVEL;
	}
	if (BitRate!=m_BitRate) {
		m_BitRate=BitRate;
		Updated|=STATISTIC_BITRATE;
	}
	if (StreamRemain!=m_StreamRemain) {
		m_StreamRemain=StreamRemain;
		Updated|=STATISTIC_STREAMREMAIN;
	}
	return Updated;
}


void CCoreEngine::ResetErrorCount()
{
	m_DtvEngine.m_TsPacketParser.ResetErrorPacketCount();
	m_ErrorPacketCount=0;
	m_ContinuityErrorPacketCount=0;
	m_DtvEngine.m_TsDescrambler.ResetScramblePacketCount();
	m_ScramblePacketCount=0;
}


/*
bool CCoreEngine::UpdateEpgDataInfo()
{
	WORD ServiceID;

	SAFE_DELETE(m_pEpgDataInfo);
	SAFE_DELETE(m_pEpgDataInfoNext);
	if (!m_DtvEngine.GetServiceID(&ServiceID))
		return false;
	m_pEpgDataInfo=m_DtvEngine.GetEpgDataInfo(ServiceID,false);
	m_pEpgDataInfoNext=m_DtvEngine.GetEpgDataInfo(ServiceID,true);
	return true;
}


const CEpgDataInfo *CCoreEngine::GetEpgDataInfo(bool fNext) const
{
	if (fNext)
		return m_pEpgDataInfoNext;
	return m_pEpgDataInfo;
}
*/
