#ifndef CORE_ENGINE_H
#define CORE_ENGINE_H


#include "DtvEngine.h"


class CCoreEngine : public CBonErrorHandler {
public:
	enum DriverType {
		DRIVER_UNKNOWN,
		DRIVER_UDP,
		DRIVER_HDUS
	};
	enum { MAX_VOLUME=100 };
private:
	TCHAR m_szDriverFileName[MAX_PATH];
	HMODULE m_hDriverLib;
	DriverType m_DriverType;
	bool m_fDescramble;
	CCardReader::ReaderType m_CardReaderType;
	CDtvEngineHandler *m_pDtvEngineHandler;
	WORD m_EventID;
	DWORD m_ErrorPacketCount;
	DWORD m_ContinuityErrorPacketCount;
	DWORD m_ScramblePacketCount;
	float m_SignalLevel;
	DWORD m_BitRate;
	DWORD m_StreamRemain;
	DWORD m_PacketBufferUsedCount;
	/*
	CEpgDataInfo *m_pEpgDataInfo;
	CEpgDataInfo *m_pEpgDataInfoNext;
	*/
public:
	CCoreEngine();
	~CCoreEngine();
	void Close();
	bool BuildDtvEngine(CDtvEngineHandler *pDtvEngineHandler);
	bool SetDriverFileName(LPCTSTR pszFileName);
	LPCTSTR GetDriverFileName() const { return m_szDriverFileName; }
	bool LoadDriver();
	bool UnloadDriver();
	bool IsDriverLoaded() const { return m_hDriverLib!=NULL; }
	bool OpenDriver();
	bool CloseDriver();
	bool IsDriverOpen() const;
	bool OpenBcasCard();
	bool IsBcasCardOpen() const;
	bool IsBuildComplete() const;
	bool SetDescramble(bool fDescramble);
	bool GetDescramble() const { return m_fDescramble; }
	bool SetCardReaderType(CCardReader::ReaderType Type);
	CCardReader::ReaderType GetCardReaderType() const { return m_CardReaderType; }
	DriverType GetDriverType() const { return m_DriverType; }
	bool IsUDPDriver() const { return m_DriverType==DRIVER_UDP; }
	enum {
		STATUS_EVENTID=0x00000001UL,
	};
	DWORD UpdateAsyncStatus();
	enum {
		STATISTIC_ERRORPACKETCOUNT				=0x00000001UL,
		STATISTIC_CONTINUITYERRORPACKETCOUNT	=0x00000002UL,
		STATISTIC_SCRAMBLEPACKETCOUNT			=0x00000004UL,
		STATISTIC_SIGNALLEVEL					=0x00000008UL,
		STATISTIC_BITRATE						=0x00000010UL,
		STATISTIC_STREAMREMAIN					=0x00000020UL,
		STATISTIC_PACKETBUFFERRATE				=0x00000040UL
	};
	DWORD UpdateStatistics();
	DWORD GetErrorPacketCount() const { return m_ErrorPacketCount; }
	DWORD GetContinuityErrorPacketCount() const { return m_ContinuityErrorPacketCount; }
	DWORD GetScramblePacketCount() const { return m_ScramblePacketCount; }
	void ResetErrorCount();
	float GetSignalLevel() const { return m_SignalLevel; }
	DWORD GetBitRate() const { return m_BitRate; }
	float GetBitRateFloat() const { return (float)m_BitRate/(float)(1024*1024); }
	DWORD GetStreamRemain() const { return m_StreamRemain; }
	int GetPacketBufferUsedPercentage();
	/*
	bool UpdateEpgDataInfo();
	const CEpgDataInfo *GetEpgDataInfo(bool fNext=false) const;
	*/
//private:
	CDtvEngine m_DtvEngine;
};


#endif
