#include "stdafx.h"
#include "DtvEngine.h"
#include "RecTest.h"
#include "AppMain.h"
#include "Menu.h"
#include "ChannelManager.h"
#include "ResidentManager.h"
#include "DriverManager.h"
#include "StatusView.h"
#include "Record.h"
#include "Plugin.h"
#include "TvRock.h"
#include "StatusOptions.h"
#include "ColorScheme.h"
#include "UdpOptions.h"
#include "Settings.h"
#include "CommandLine.h"
#include "Logger.h"
#include "ToolTip.h"
#include "StreamInfo.h"
#include "MiscDialog.h"
#include "DialogUtil.h"
#include "DrawUtil.h"
#include "resource.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


#define MAIN_WINDOW_CLASS APP_NAME TEXT(" Window")

#define TITLE_TEXT APP_NAME

#define CAPTION_WIDTH 24

static HINSTANCE hInst;
static CAppMain AppMain;
static CCoreEngine CoreEngine;
static CMainWindow MainWindow;
static CStatusView StatusView;
static CMainMenu MainMenu;
static CToolTip ToolTip;
//static CHtmlHelp HtmlHelpClass;

static TCHAR szDriverFileName[MAX_PATH];
//static bool fNewHDUSDriver=true;
static bool fIncrementUDPPort=true;

static CCommandLineParser CmdLineParser;

static CChannelManager ChannelManager;
static CResidentManager ResidentManager;
static CDriverManager DriverManager;

static CStatusOptions StatusOptions(&StatusView);
static CColorSchemeOptions ColorSchemeOptions;
static CRecordOptions RecordOptions;
static CRecordManager RecordManager;
static CUdpOptions UdpOptions;
static CPluginList PluginList;
static CPluginOptions PluginOptions(&PluginList);
static CLogger Logger;

static CStreamInfo StreamInfo;

static CTvRockDTVTarget TvRockDTVTarget;

static bool fDescrambleUseSSE2=false;
static bool fEnableEmmProcess=false;
static bool fDescrambleCurServiceOnly=false;
static bool fNoTaskBar=false;
static bool fRestoreChannel=false;
static struct {
	int Space;
	int Channel;
	int ServiceID;
	TCHAR szDriverName[MAX_PATH];
} RestoreChannelInfo;

static bool fSnapAtWindowEdge=true;
static int SnapAtWindowEdgeMargin=8;
enum {
	INITIALPOS_LAST,
	INITIALPOS_LOWERLEFT,
	INITIALPOS_LOWERRIGHT,
	INITIALPOS_UPPERLEFT,
	INITIALPOS_UPPERRIGHT
};
static int InitialPos=INITIALPOS_LOWERRIGHT;

static bool fWheelChannelReverse=false;
static unsigned int WheelChannelDelay=1000;
static bool fWheelChannelChanging=false;
static bool fFunctionKeyChangeChannel=true;
static bool fDigitKeyChangeChannel=true;
static bool fNumPadChangeChannel=true;




bool CAppMain::Initialize()
{
	TCHAR szModuleFileName[MAX_PATH];

	::GetModuleFileName(NULL,szModuleFileName,MAX_PATH);
	if (CmdLineParser.m_szIniFileName[0]=='\0') {
		::lstrcpy(m_szIniFileName,szModuleFileName);
		::PathRenameExtension(m_szIniFileName,TEXT(".ini"));
	} else {
		if (::PathIsFileSpec(CmdLineParser.m_szIniFileName)) {
			::lstrcpy(m_szIniFileName,szModuleFileName);
			::lstrcpy(::PathFindFileName(m_szIniFileName),CmdLineParser.m_szIniFileName);
		} else {
			::lstrcpy(m_szIniFileName,CmdLineParser.m_szIniFileName);
		}
	}
	::lstrcpy(m_szDefaultChannelFileName,szModuleFileName);
	::PathRenameExtension(m_szDefaultChannelFileName,TEXT(".ch2"));
	::lstrcpy(m_szChannelSettingFileName,szModuleFileName);
	::lstrcpy(::PathFindFileName(m_szChannelSettingFileName),TEXT("TVTest.ch.ini"));
	/*
	// TvftHgƂĈ
	if (!PathFileExists(m_szDefaultChannelFileName)) {
		TCHAR szSample[MAX_PATH];

		::lstrcpy(szSample,m_szDefaultChannelFileName);
		::PathRenameExtension(szSample,TEXT(".sample.ch"));
		::CopyFile(szSample,m_szDefaultChannelFileName,TRUE);
	}
	*/
	m_fFirstExecute=!::PathFileExists(m_szIniFileName);
	if (!m_fFirstExecute) {
		Logger.AddLog(TEXT("ݒǂݍł܂..."));
		LoadSettings();
	}
	return true;
}


bool CAppMain::Finalize()
{
	Logger.AddLog(TEXT("ݒۑĂ܂..."));
	SaveSettings();
	//SaveChannelSettings();
	return true;
}


HINSTANCE CAppMain::GetResourceInstance() const
{
	return hInst;
}


HINSTANCE CAppMain::GetInstance() const
{
	return hInst;
}


bool CAppMain::GetAppDirectory(LPTSTR pszDirectory) const
{
	if (::GetModuleFileName(NULL,pszDirectory,MAX_PATH)==0)
		return false;
	CFilePath FilePath(pszDirectory);
	FilePath.RemoveFileName();
	FilePath.GetPath(pszDirectory);
	return true;
}


bool CAppMain::AddLog(LPCTSTR pszText, ...)
{
	va_list Args;

	va_start(Args,pszText);
	bool fOK=Logger.AddLogV(pszText,Args);
	va_end(Args);
	return fOK;
}


bool CAppMain::InitializeChannel()
{
	bool fUDPDriver=CoreEngine.IsUDPDriver();
	CFilePath ChannelFilePath;

	ChannelManager.Clear();
	ChannelManager.MakeDriverTuningSpaceList(&CoreEngine.m_DtvEngine.m_BonSrcDecoder);
	if (!fUDPDriver) {
		ChannelFilePath.SetPath(CoreEngine.GetDriverFileName());
		if (!ChannelFilePath.HasDirectory()) {
			TCHAR szDir[MAX_PATH];

			GetAppDirectory(szDir);
			ChannelFilePath.SetDirectory(szDir);
		}
		ChannelFilePath.SetExtension(TEXT(".ch2"));
		if (!ChannelFilePath.IsExists())
			ChannelFilePath.SetExtension(TEXT(".ch"));
	} else {
		if (szDriverFileName[0]!='\0'
				&& ::lstrcmpi(::PathFindFileName(szDriverFileName),
							  TEXT("BonDriver_UDP.dll"))!=0) {
			ChannelFilePath.SetPath(szDriverFileName);
			ChannelFilePath.SetExtension(TEXT(".ch2"));
			if (!ChannelFilePath.IsExists())
				ChannelFilePath.SetExtension(TEXT(".ch"));
		}
	}
	if (ChannelFilePath.GetPath()[0]=='\0' || !ChannelFilePath.IsExists()) {
		ChannelFilePath.SetPath(m_szDefaultChannelFileName);
		if (!ChannelFilePath.IsExists()) {
			ChannelFilePath.SetExtension(TEXT(".ch"));
		}
	}
	if (ChannelManager.LoadChannelList(ChannelFilePath.GetPath()))
		Logger.AddLog(TEXT("`lݒ \"%s\" ǂݍ݂܂B"),
												ChannelFilePath.GetFileName());
	TCHAR szFileName[MAX_PATH];
	bool fLoadChannelSettings=true;
	if (!fUDPDriver) {
		::lstrcpy(szFileName,CoreEngine.GetDriverFileName());
	} else {
		if (::lstrcmpi(::PathFindFileName(szDriverFileName),TEXT("BonDriver_UDP.dll"))!=0) {
			::lstrcpy(szFileName,szDriverFileName);
		} else {
			fLoadChannelSettings=false;
		}
	}
	if (fLoadChannelSettings)
		ChannelManager.LoadChannelSettings(m_szChannelSettingFileName,szFileName);
	ChannelManager.SetUseDriverChannelList(fUDPDriver);
	/*
	ChannelManager.SetCurrentSpace(
		(!fUDPDriver && ChannelManager.GetAllChannelList()->NumChannels()>0)?
											CChannelManager::SPACE_ALL:0);
	*/
	ChannelManager.SetCurrentChannel(0,fUDPDriver?0:-1);
	ChannelManager.SetCurrentServiceID(0);
	SetTuningSpaceMenu();
	SetChannelMenu();
	ClearMenu(MainMenu.GetSubMenu(CMainMenu::SUBMENU_SERVICE));

	RestoreChannelInfo.Space=-1;
	RestoreChannelInfo.Channel=-1;
	RestoreChannelInfo.ServiceID=-1;
	if (fRestoreChannel && !fUDPDriver) {
		CSettings Setting;

		if (Setting.Open(m_szIniFileName,TEXT("LastChannel"),CSettings::OPEN_READ)) {
			TCHAR szDriverName[MAX_PATH],szName[32+MAX_PATH];

			::lstrcpy(szDriverName,::PathFindFileName(CoreEngine.GetDriverFileName()));
			*::PathFindExtension(szDriverName)='\0';
			::wsprintf(szName,TEXT("Space_%s"),szDriverName);
			Setting.Read(szName,&RestoreChannelInfo.Space);
			::wsprintf(szName,TEXT("ChannelIndex_%s"),szDriverName);
			Setting.Read(szName,&RestoreChannelInfo.Channel);
			::wsprintf(szName,TEXT("ServiceID_%s"),szDriverName);
			Setting.Read(szName,&RestoreChannelInfo.ServiceID);
			Setting.Close();
		}
	}
	return true;
}


bool CAppMain::UpdateChannelList(const CTuningSpaceList *pList)
{
	bool fUDPDriver=CoreEngine.IsUDPDriver();

	ChannelManager.SetTuningSpaceList(pList);
	ChannelManager.SetUseDriverChannelList(fUDPDriver);
	ChannelManager.SetCurrentChannel(
		(!fUDPDriver && ChannelManager.GetAllChannelList()->NumChannels()>0)?
												CChannelManager::SPACE_ALL:0,
		fUDPDriver?0:-1);
	ChannelManager.SetCurrentServiceID(0);
	SetTuningSpaceMenu();
	SetChannelMenu();
	ClearMenu(MainMenu.GetSubMenu(CMainMenu::SUBMENU_SERVICE));
	return true;
}


bool CAppMain::SaveChannelSettings()
{
	/*
	if (!CoreEngine.IsDriverLoaded() || CoreEngine.IsUDPDriver())
		return true;
	return ChannelManager.SaveChannelSettings(m_szChannelSettingFileName,
											  CoreEngine.GetDriverFileName());
	*/
	return false;
}


void CAppMain::SetTuningSpaceMenu(HMENU hmenu)
{
	TCHAR szText[MAX_PATH*2];
	int Length;
	int i;

	ClearMenu(hmenu);
	if ((!CoreEngine.IsUDPDriver())
			&& ChannelManager.GetAllChannelList()->NumChannels()>0)
		::AppendMenu(hmenu,MFT_STRING | MFS_ENABLED,CM_SPACE_ALL,TEXT("&A: ׂ"));
	const CTuningSpaceList *pTuningSpaceList=ChannelManager.GetDriverTuningSpaceList();
	for (i=0;i<pTuningSpaceList->NumSpaces();i++) {
		LPCTSTR pszName=pTuningSpaceList->GetTuningSpaceName(i);

		Length=::wsprintf(szText,TEXT("&%d: "),i);
		CopyToMenuText(pszName!=NULL?pszName:TEXT("???"),
					   szText+Length,lengthof(szText)-Length);
		::AppendMenu(hmenu,MFT_STRING | MFS_ENABLED,CM_SPACE_FIRST+i,szText);
	}
	::CheckMenuRadioItem(hmenu,CM_SPACE_ALL,CM_SPACE_ALL+pTuningSpaceList->NumSpaces(),
				CM_SPACE_FIRST+ChannelManager.GetCurrentSpace(),MF_BYCOMMAND);
	::AppendMenu(hmenu,MFT_SEPARATOR,0,NULL);
	int CurDriver=-1;
	for (i=0;i<DriverManager.NumDrivers();i++) {
		const CDriverInfo *pDriverInfo=DriverManager.GetDriverInfo(i);

		CopyToMenuText(pDriverInfo->GetFileName(),szText,lengthof(szText));
		::AppendMenu(hmenu,MFT_STRING | MFS_ENABLED,CM_DRIVER_FIRST+i,szText);
		if (::lstrcmpi(pDriverInfo->GetFileName(),CoreEngine.GetDriverFileName())==0)
			CurDriver=i;
	}
	if (CurDriver<0) {
		CopyToMenuText(CoreEngine.GetDriverFileName(),szText,lengthof(szText));
		::AppendMenu(hmenu,MFT_STRING | MFS_ENABLED,CM_DRIVER_FIRST+i,szText);
		CurDriver=i++;
	}
	if (CoreEngine.IsDriverLoaded())
		::CheckMenuRadioItem(hmenu,CM_DRIVER_FIRST,CM_DRIVER_FIRST+i-1,
							 CM_DRIVER_FIRST+CurDriver,MF_BYCOMMAND);
}


void CAppMain::SetTuningSpaceMenu()
{
	SetTuningSpaceMenu(MainMenu.GetSubMenu(CMainMenu::SUBMENU_SPACE));
}


void CAppMain::SetChannelMenu(HMENU hmenu)
{
	const CChannelList *pList=ChannelManager.GetCurrentChannelList();
	int i;
	TCHAR szText[MAX_CHANNEL_NAME+4];

	ClearMenu(hmenu);
	if (pList==NULL)
		return;
	bool fControlKeyID=pList->HasRemoteControlKeyID();
	for (i=0;i<pList->NumChannels();i++) {
		wsprintf(szText,TEXT("%d: %s"),
				 fControlKeyID?pList->GetChannelNo(i):i+1,pList->GetName(i));
		AppendMenu(hmenu,MFT_STRING | MFS_ENABLED
				| (i!=0 && i%12==0?MF_MENUBREAK:0),CM_CHANNEL_FIRST+i,szText);
	}
	if (ChannelManager.GetCurrentChannel()>=0)
		MainMenu.CheckRadioItem(CM_CHANNEL_FIRST,
			CM_CHANNEL_FIRST+pList->NumChannels()-1,
			CM_CHANNEL_FIRST+ChannelManager.GetCurrentChannel());
}


void CAppMain::SetChannelMenu()
{
	SetChannelMenu(MainMenu.GetSubMenu(CMainMenu::SUBMENU_CHANNEL));
}


bool CAppMain::UpdateChannelMenu()
{
	SetChannelMenu();
	return true;
}


const CChannelInfo *CAppMain::GetCurrentChannelInfo() const
{
	return ChannelManager.GetCurrentChannelInfo();
}


bool CAppMain::SetChannel(int Space,int Channel,int ServiceID/*=-1*/)
{
	const CChannelInfo *pPrevChInfo=ChannelManager.GetCurrentRealChannelInfo();
	int OldSpace=ChannelManager.GetCurrentSpace(),OldChannel=ChannelManager.GetCurrentChannel();

	if (!ChannelManager.SetCurrentChannel(Space,Channel))
		return false;
	const CChannelInfo *pChInfo=ChannelManager.GetCurrentRealChannelInfo();
	if (pChInfo==NULL) {
		ChannelManager.SetCurrentChannel(OldSpace,OldChannel);
		return false;
	}
	MainWindow.OpenTuner();
	if (pPrevChInfo==NULL
			|| pChInfo->GetSpace()!=pPrevChInfo->GetSpace()
			|| pChInfo->GetChannelIndex()!=pPrevChInfo->GetChannelIndex()) {
		if (ServiceID<=0 && pChInfo->GetServiceID()>0)
			ServiceID=pChInfo->GetServiceID();

		LPCTSTR pszTuningSpace=ChannelManager.GetDriverTuningSpaceList()->GetTuningSpaceName(pChInfo->GetSpace());
		AddLog(TEXT("BonDriverɃ`lύXv܂B(`[jO %d[%s] / Ch %d / Sv %d)"),
			   pChInfo->GetSpace(),pszTuningSpace!=NULL?pszTuningSpace:TEXT("???"),
			   pChInfo->GetChannelIndex(),ServiceID);

		if (!CoreEngine.m_DtvEngine.SetChannel(
							pChInfo->GetSpace(),pChInfo->GetChannelIndex(),
							ServiceID>0?ServiceID:CDtvEngine::SID_INVALID)) {
			AddLog(CoreEngine.m_DtvEngine.GetLastErrorText());
			MainWindow.PopupErrorMessage(CoreEngine.m_DtvEngine.GetLastErrorText());
			ChannelManager.SetCurrentChannel(OldSpace,OldChannel);
			return false;
		}
		ChannelManager.SetCurrentServiceID(ServiceID);
		TCHAR szTip[256];
		::wsprintf(szTip,APP_NAME TEXT(": %s"),pChInfo->GetName());
		ResidentManager.SetTipText(szTip);
		TvRockDTVTarget.OnEvent(CTvRockDTVTarget::EVENT_CHANNELCHANGE);
		PluginList.SendChannelChangeEvent();
	} else {
		ChannelManager.SetCurrentServiceID(ServiceID);
		if (ServiceID>0) {
			SetServiceByID(ServiceID);
		} else {
			SetServiceByID(pChInfo->GetServiceID());
		}
	}
	StatusView.UpdateItem(STATUS_ITEM_CHANNEL);
	StatusView.UpdateItem(STATUS_ITEM_TUNER);
	SetChannelMenu();
	MainMenu.CheckRadioItem(CM_SPACE_ALL,CM_SPACE_ALL+ChannelManager.NumSpaces(),
		CM_SPACE_FIRST+ChannelManager.GetCurrentSpace());
	MainMenu.CheckRadioItem(CM_CHANNEL_FIRST,
		CM_CHANNEL_FIRST+ChannelManager.GetCurrentRealChannelList()->NumChannels()-1,
		CM_CHANNEL_FIRST+ChannelManager.GetCurrentChannel());
	return true;
}


bool CAppMain::FollowChannelChange(WORD TransportStreamID,WORD ServiceID)
{
	Logger.AddLog(TEXT("`lύXm܂B(TSID %d / SID %d)"),
				  TransportStreamID,ServiceID);

	const CChannelList *pChannelList;
	const CChannelInfo *pChannelInfo;
	int i,j;
	int Space,Channel;
	bool fFinded=false;

	pChannelList=ChannelManager.GetCurrentRealChannelList();
	if (pChannelList!=NULL) {
		for (i=0;i<pChannelList->NumChannels();i++) {
			pChannelInfo=pChannelList->GetChannelInfo(i);
			if (pChannelInfo->GetTransportStreamID()==TransportStreamID
					&& pChannelInfo->GetServiceID()==ServiceID) {
				Space=ChannelManager.GetCurrentSpace();
				Channel=i;
				fFinded=true;
				break;
			}
		}
	} else {
		for (i=0;i<ChannelManager.NumSpaces();i++) {
			pChannelList=ChannelManager.GetChannelList(i);
			for (j=0;j<pChannelList->NumChannels();j++) {
				pChannelInfo=pChannelList->GetChannelInfo(j);
				if (pChannelInfo->GetTransportStreamID()==TransportStreamID
						&& pChannelInfo->GetServiceID()==ServiceID) {
					Space=i;
					Channel=j;
					fFinded=true;
					goto End;
				}
			}
		}
	End:
		;
	}
	if (!fFinded) {
		Logger.AddLog(TEXT("mTSID/SIDłB"));
		return false;
	}
	if (!ChannelManager.SetCurrentChannel(Space,Channel))
		return false;
	Logger.AddLog(TEXT("Ch %d(%s)"),
				  pChannelInfo->GetChannelIndex(),pChannelInfo->GetName());
	ChannelManager.SetCurrentServiceID(0);
	TvRockDTVTarget.OnEvent(CTvRockDTVTarget::EVENT_CHANNELCHANGE);
	PluginList.SendChannelChangeEvent();
	StatusView.UpdateItem(STATUS_ITEM_CHANNEL);
	StatusView.UpdateItem(STATUS_ITEM_TUNER);
	SetChannelMenu();
	MainMenu.CheckRadioItem(CM_SPACE_ALL,CM_SPACE_ALL+ChannelManager.NumSpaces(),
							CM_SPACE_FIRST+Space);
	MainMenu.CheckRadioItem(CM_CHANNEL_FIRST,
		CM_CHANNEL_FIRST+ChannelManager.GetCurrentRealChannelList()->NumChannels()-1,
		CM_CHANNEL_FIRST+Channel);
	return true;
}


bool CAppMain::SetService(int Service)
{
	int NumServices=CoreEngine.m_DtvEngine.m_ProgManager.GetServiceNum();

	if (Service<0 || Service>=NumServices
			|| !CoreEngine.m_DtvEngine.SetService(Service))
		return false;
	WORD ServiceID=0;
	CoreEngine.m_DtvEngine.GetServiceID(&ServiceID);
	AddLog(TEXT("T[rXύX܂B(%d: SID %d)"),Service,ServiceID);
	MainMenu.CheckRadioItem(CM_SERVICE_FIRST,CM_SERVICE_FIRST+NumServices-1,
							CM_SERVICE_FIRST+Service);
	TvRockDTVTarget.OnEvent(CTvRockDTVTarget::EVENT_SERVICECHANGE);
	PluginList.SendServiceChangeEvent();
	return true;
}


bool CAppMain::SetServiceByIndex(int Service)
{
	if (!SetService(Service))
		return false;
	WORD ServiceID=0;
	CoreEngine.m_DtvEngine.GetServiceID(&ServiceID);
	ChannelManager.SetCurrentServiceID(Service);
	return true;
}


bool CAppMain::SetServiceByID(WORD ServiceID,int *pServiceIndex/*=NULL*/)
{
	if (ServiceID==0)
		return SetService(0);

	AddLog(TEXT("T[rXIĂ܂(SID %d)..."),ServiceID);
	WORD Index=CoreEngine.m_DtvEngine.m_ProgManager.GetServiceIndexByID(ServiceID);
	if (Index!=0xFFFF) {
		if (SetService(Index)) {
			if (pServiceIndex!=NULL)
				*pServiceIndex=Index;
			return true;
		}
		return false;
	}
	AddLog(TEXT("YT[rXt܂Bw肳ꂽSIDPMTɂ܂B"));
	ChannelManager.SetCurrentServiceID(ServiceID);
	return false;
}


bool CAppMain::Set1SegService()
{
	if (!CoreEngine.m_DtvEngine.Set1SegService(true))
		return false;
	int CurService=CoreEngine.m_DtvEngine.GetService();
	BYTE StreamType;
	if (CoreEngine.m_DtvEngine.m_ProgManager.GetVideoStreamType(&StreamType,CurService)
			&& StreamType == 0x1B) {
		SetService(CurService);
	}
	return true;
}


bool CAppMain::SetDriver(LPCTSTR pszFileName)
{
	HCURSOR hcurOld=::SetCursor(::LoadCursor(NULL,IDC_WAIT));
	bool fOK;

	SaveChannelSettings();
	CoreEngine.m_DtvEngine.SetTracer(&StatusView);
	CoreEngine.SetDriverFileName(pszFileName);
	fOK=CoreEngine.LoadDriver();
	if (fOK) {
		Logger.AddLog(TEXT("%s ǂݍ݂܂B"),CoreEngine.GetDriverFileName());
		fOK=CoreEngine.OpenDriver();
		if (fOK) {
			AppMain.InitializeChannel();
			int i=ChannelManager.GetCurrentChannelList()->Find(
				CoreEngine.m_DtvEngine.m_BonSrcDecoder.GetCurSpace(),
				CoreEngine.m_DtvEngine.m_BonSrcDecoder.GetCurChannel(),0);
			if (i>=0)
				MainWindow.PostCommand(CM_CHANNEL_FIRST+i);
			::SetCursor(hcurOld);
			TvRockDTVTarget.OnEvent(CTvRockDTVTarget::EVENT_DRIVERCHANGE);;
			PluginList.SendDriverChangeEvent();
		} else {
			::SetCursor(hcurOld);
			MainWindow.PopupErrorMessage(TEXT("BonDriveȑł܂B"));
		}
	} else {
		TCHAR szMessage[MAX_PATH+64];

		::SetCursor(hcurOld);
		::wsprintf(szMessage,TEXT("\"%s\" [hł܂B"),pszFileName);
		MainWindow.PopupErrorMessage(szMessage);
	}
	CoreEngine.m_DtvEngine.SetTracer(NULL);
	StatusView.SetSingleText(NULL);
	return fOK;
}


bool CAppMain::UpdateDriverMenu()
{
	SetTuningSpaceMenu();
	return true;
}


HMENU CAppMain::CreateTunerSelectMenu()
{
	HMENU hmenu=::CreatePopupMenu(),hmenuSpace;
	const CChannelList *pChannelList;
	const CTuningSpaceList *pTuningSpaceList;
	int Command,FirstCommand;
	int i,j;
	LPCTSTR pszName;
	TCHAR szText[MAX_PATH*2];
	int Length;

	Command=CM_SPACE_CHANNEL_FIRST;
	/*
	if ((!CoreEngine.IsUDPDriver() || pNetworkRemocon==NULL)
			&& ChannelManager.GetAllChannelList()->NumChannels()>0) {
		hmenuSpace=::CreatePopupMenu();
		pChannelList=ChannelManager.GetAllChannelList();
		for (i=0;i<pChannelList->NumChannels();i++) {
			pszName=pChannelList->GetName(i);
			CopyToMenuText(pszName!=NULL?pszName:TEXT("???"),szText,lengthof(szText));
			::AppendMenu(hmenuSub,MFT_STRING | MFS_ENABLED,Command++,szText);
		}
		::AppendMenu(hmenu,MFT_POPUP | MFS_ENABLED,
					 reinterpret_cast<UINT_PTR>(hmenuSpace),TEXT("&0: ׂ"));
	}
	*/
	pTuningSpaceList=ChannelManager.GetDriverTuningSpaceList();
	for (i=0;i<pTuningSpaceList->NumSpaces();i++) {
		pChannelList=ChannelManager.GetChannelList(i);
		hmenuSpace=::CreatePopupMenu();
		FirstCommand=Command;
		bool fHasControlKeyID=pChannelList->HasRemoteControlKeyID();
		for (j=0;j<pChannelList->NumChannels();j++) {
			const CChannelInfo *pChannelInfo=pChannelList->GetChannelInfo(j);

			Length=::wsprintf(szText,TEXT("%d : "),
							  fHasControlKeyID?pChannelInfo->GetChannelNo():j+1);
			pszName=pChannelInfo->GetName();
			CopyToMenuText(pszName!=NULL?pszName:TEXT("???"),
						   szText+Length,lengthof(szText)-Length);
			::AppendMenu(hmenuSpace,MFT_STRING | MFS_ENABLED,Command++,szText);
		}
		if (ChannelManager.GetCurrentSpace()==i
				&& ChannelManager.GetCurrentChannel()>=0) {
			::CheckMenuRadioItem(hmenuSpace,FirstCommand,Command-1,
								 FirstCommand+ChannelManager.GetCurrentChannel(),
								 MF_BYCOMMAND);
		}
		Length=::wsprintf(szText,TEXT("&%d: "),i+1);
		pszName=pTuningSpaceList->GetTuningSpaceName(i);
		CopyToMenuText(pszName!=NULL?pszName:TEXT("???"),
					   szText+Length,lengthof(szText)-Length);
		::AppendMenu(hmenu,MF_POPUP | MFS_ENABLED,
					 reinterpret_cast<UINT_PTR>(hmenuSpace),szText);
	}
	if (pTuningSpaceList->NumSpaces()>0)
		AppendMenu(hmenu,MFT_SEPARATOR,0,NULL);
	//int CurDriver=-1;
	for (i=0;i<DriverManager.NumDrivers();i++) {
		CDriverInfo *pDriverInfo=DriverManager.GetDriverInfo(i);

		if (::lstrcmpi(pDriverInfo->GetFileName(),CoreEngine.GetDriverFileName())==0) {
			/*
			CurDriver=i;
			CopyToMenuText(pDriverInfo->GetFileName(),szText,lengthof(szText));
			::AppendMenu(hmenu,MFT_STRING | MFS_ENABLED | MFS_CHECKED,
						 CM_DRIVER_FIRST+i,szText);
			*/
			continue;
		}
		if (pDriverInfo->LoadTuningSpaceList(
				!::PathMatchSpec(pDriverInfo->GetFileName(),TEXT("BonDriver_Spinel*.dll")))) {
			HMENU hmenuDriver=::CreatePopupMenu();

			pTuningSpaceList=pDriverInfo->GetTuningSpaceList();
			for (j=0;j<pTuningSpaceList->NumSpaces();j++) {
				pChannelList=pTuningSpaceList->GetChannelList(j);
				if (pTuningSpaceList->NumSpaces()>1)
					hmenuSpace=::CreatePopupMenu();
				else
					hmenuSpace=hmenuDriver;
				bool fHasControlKeyID=pChannelList->HasRemoteControlKeyID();
				for (int k=0;k<pChannelList->NumChannels();k++) {
					const CChannelInfo *pChannelInfo=pChannelList->GetChannelInfo(k);

					Length=::wsprintf(szText,TEXT("%d: "),
							fHasControlKeyID?pChannelInfo->GetChannelNo():k+1);
					pszName=pChannelInfo->GetName();
					CopyToMenuText(pszName!=NULL?pszName:TEXT("???"),
								   szText+Length,lengthof(szText)-Length);
					::AppendMenu(hmenuSpace,MFT_STRING | MFS_ENABLED,Command++,szText);
				}
				if (hmenuSpace!=hmenuDriver) {
					pszName=pTuningSpaceList->GetTuningSpaceName(j);
					Length=::wsprintf(szText,TEXT("&%d: "),j+1);
					CopyToMenuText(pszName!=NULL?pszName:TEXT("???"),
								   szText+Length,lengthof(szText)-Length);
					::AppendMenu(hmenuDriver,MF_POPUP | MFS_ENABLED,
								 reinterpret_cast<UINT_PTR>(hmenuSpace),szText);
				}
			}
			if (pDriverInfo->GetTunerName()!=NULL) {
				TCHAR szTemp[lengthof(szText)];

				::wnsprintf(szTemp,lengthof(szTemp),TEXT("%s [%s]"),
							pDriverInfo->GetTunerName(),
							pDriverInfo->GetFileName());
				CopyToMenuText(szTemp,szText,lengthof(szText));
			} else {
				CopyToMenuText(pDriverInfo->GetFileName(),szText,lengthof(szText));
			}
			::AppendMenu(hmenu,MF_POPUP | MFS_ENABLED,
						 reinterpret_cast<UINT_PTR>(hmenuDriver),szText);
		} else {
			CopyToMenuText(pDriverInfo->GetFileName(),szText,lengthof(szText));
			::AppendMenu(hmenu,MFT_STRING | MFS_ENABLED,CM_DRIVER_FIRST+i,szText);
		}
	}
	/*
	if (CurDriver<0) {
		CopyToMenuText(CoreEngine.GetDriverFileName(),szText,lengthof(szText));
		::AppendMenu(hmenu,MFT_STRING | MFS_ENABLED | MFS_CHECKED,
					 CM_DRIVER_FIRST+i,szText);
	}
	*/
	return hmenu;
}


bool CAppMain::ProcessTunerSelectMenu(int Command)
{
	if (Command<CM_SPACE_CHANNEL_FIRST || Command>CM_SPACE_CHANNEL_LAST)
		return false;

	const CTuningSpaceList *pTuningSpaceList;
	const CChannelList *pChannelList;
	int CommandBase;
	int i,j;

	CommandBase=CM_SPACE_CHANNEL_FIRST;
	/*
	if (!CoreEngine.IsUDPDriver() || pNetworkRemocon==NULL) {
		pChannelList=ChannelManager.GetAllChannelList();
		if (pChannelList->NumChannels()>0) {
			if (Command-CommandBase<pChannelList->NumChannels())
				return SetChannel(-1,Command-CommandBase);
			CommandBase+=pChannelList->NumChannels();
		}
	}
	*/
	pTuningSpaceList=ChannelManager.GetDriverTuningSpaceList();
	for (int i=0;i<pTuningSpaceList->NumSpaces();i++) {
		pChannelList=ChannelManager.GetChannelList(i);
		if (Command-CommandBase<pChannelList->NumChannels())
			return SetChannel(i,Command-CommandBase);
		CommandBase+=pChannelList->NumChannels();
	}
	for (i=0;i<DriverManager.NumDrivers();i++) {
		const CDriverInfo *pDriverInfo=DriverManager.GetDriverInfo(i);

		if (::lstrcmpi(pDriverInfo->GetFileName(),CoreEngine.GetDriverFileName())==0)
			continue;
		if (pDriverInfo->IsTuningSpaceListLoaded()) {
			pTuningSpaceList=pDriverInfo->GetTuningSpaceList();
			for (j=0;j<pTuningSpaceList->NumSpaces();j++) {
				pChannelList=pTuningSpaceList->GetChannelList(j);
				if (Command-CommandBase<pChannelList->NumChannels()) {
					if (!SetDriver(pDriverInfo->GetFileName()))
						return false;
					::lstrcpy(szDriverFileName,pDriverInfo->GetFileName());
					return SetChannel(j,Command-CommandBase);
				}
				CommandBase+=pChannelList->NumChannels();
			}
		}
	}
	return false;
}


bool CAppMain::LoadSettings()
{
	CSettings Setting;

	if (Setting.Open(m_szIniFileName,TEXT("Settings"),CSettings::OPEN_READ)) {
		int Value;
		TCHAR szText[MAX_PATH];
		int Left,Top,Width,Height;
		bool f;

		MainWindow.GetPosition(&Left,&Top,&Width,&Height);
		Setting.Read(TEXT("WindowLeft"),&Left);
		Setting.Read(TEXT("WindowTop"),&Top);
		Setting.Read(TEXT("WindowWidth"),&Width);
		//Setting.Read(TEXT("WindowHeight"),&Height);
		MainWindow.SetPosition(Left,Top,Width,Height);
		MainWindow.MoveToMonitorInside();
		if (Setting.Read(TEXT("AlwaysOnTop"),&f))
			MainWindow.SetAlwaysOnTop(f);
		Setting.Read(TEXT("Driver"),szDriverFileName,
												lengthof(szDriverFileName));
		if (Setting.Read(TEXT("CardReader"),&Value))
			CoreEngine.SetCardReaderType((CCardReader::ReaderType)Value);
		Setting.Read(TEXT("DescrambleSSE2"),&fDescrambleUseSSE2);
		Setting.Read(TEXT("DescrambleCurServiceOnly"),&fDescrambleCurServiceOnly);
		Setting.Read(TEXT("ProcessEMM"),&fEnableEmmProcess);
		if (Setting.Read(TEXT("Resident"),&f))
			ResidentManager.SetResident(f);
		if (Setting.Read(TEXT("NoTaskBar"),&fNoTaskBar))
			ResidentManager.SetAlwaysTray(fNoTaskBar);
		if (Setting.Read(TEXT("MinimizeToTray"),&f))
			ResidentManager.SetMinimizeToTray(f);
		Setting.Read(TEXT("RestoreChannel"),&fRestoreChannel);
		Setting.Read(TEXT("SnapToWindowEdge"),&fSnapAtWindowEdge);
		Setting.Read(TEXT("InitialPos"),&InitialPos);
		Setting.Read(TEXT("ReverseWheelChannel"),&fWheelChannelReverse);
		Setting.Read(TEXT("WheelChannelDelay"),&WheelChannelDelay);
		Setting.Read(TEXT("FuncKeyChangeChannel"),&fFunctionKeyChangeChannel);
		Setting.Read(TEXT("DigitKeyChangeChannel"),&fDigitKeyChangeChannel);
		Setting.Read(TEXT("NumPadChangeChannel"),&fNumPadChangeChannel);
		if (Setting.Read(TEXT("RecOptionFileName"),szText,MAX_PATH) && szText[0]!='\0')
			RecordManager.SetFileName(szText);
		if (Setting.Read(TEXT("RecOptionExistsOperation"),&Value))
			RecordManager.SetFileExistsOperation(
								(CRecordManager::FileExistsOperation)Value);
		// Experimental options
		Setting.Read(TEXT("IncrementUDPPort"),&fIncrementUDPPort);
		RecordOptions.Read(&Setting);
		UdpOptions.Read(&Setting);
		PluginOptions.Read(&Setting);
		Logger.Read(&Setting);
		Setting.Close();
	}
	StatusOptions.Load(m_szIniFileName);
	//ColorSchemeOptions.Load(m_szIniFileName);
	TCHAR szFileName[MAX_PATH];
	::lstrcpy(szFileName,m_szIniFileName);
	::lstrcpy(::PathFindFileName(szFileName),TEXT("TVTest.ini"));
	ColorSchemeOptions.Load(szFileName);
	PluginOptions.Load(m_szIniFileName);
	return true;
}


bool CAppMain::SaveSettings()
{
	CSettings Setting;

	if (Setting.Open(m_szIniFileName,TEXT("Settings"),CSettings::OPEN_WRITE)) {
		int Left,Top,Width,Height;

		MainWindow.GetPosition(&Left,&Top,&Width,&Height);
		Setting.Write(TEXT("WindowLeft"),Left);
		Setting.Write(TEXT("WindowTop"),Top);
		Setting.Write(TEXT("WindowWidth"),Width);
		//Setting.Write(TEXT("WindowHeight"),Height);
		Setting.Write(TEXT("AlwaysOnTop"),MainWindow.GetAlwaysOnTop());
		Setting.Write(TEXT("Driver"),szDriverFileName);
		Setting.Write(TEXT("CardReader"),(int)CoreEngine.GetCardReaderType());
		Setting.Write(TEXT("DescrambleSSE2"),fDescrambleUseSSE2);
		Setting.Write(TEXT("DescrambleCurServiceOnly"),fDescrambleCurServiceOnly);
		Setting.Write(TEXT("ProcessEMM"),fEnableEmmProcess);
		Setting.Write(TEXT("Resident"),ResidentManager.GetResident());
		Setting.Write(TEXT("NoTaskBar"),fNoTaskBar);
		Setting.Write(TEXT("MinimizeToTray"),ResidentManager.GetMinimizeToTray());
		Setting.Write(TEXT("RestoreChannel"),fRestoreChannel);
		Setting.Write(TEXT("SnapToWindowEdge"),fSnapAtWindowEdge);
		Setting.Write(TEXT("InitialPos"),InitialPos);
		Setting.Write(TEXT("ReverseWheelChannel"),fWheelChannelReverse);
		Setting.Write(TEXT("WheelChannelDelay"),WheelChannelDelay);
		Setting.Write(TEXT("FuncKeyChangeChannel"),fFunctionKeyChangeChannel);
		Setting.Write(TEXT("DigitKeyChangeChannel"),fDigitKeyChangeChannel);
		Setting.Write(TEXT("NumPadChangeChannel"),fNumPadChangeChannel);
		if (RecordManager.GetFileName()!=NULL)
			Setting.Write(TEXT("RecOptionFileName"),RecordManager.GetFileName());
		Setting.Write(TEXT("RecOptionExistsOperation"),
										RecordManager.GetFileExistsOperation());
		Setting.Write(TEXT("IncrementUDPPort"),fIncrementUDPPort);
		RecordOptions.Write(&Setting);
		UdpOptions.Write(&Setting);
		PluginOptions.Write(&Setting);
		Logger.Write(&Setting);
		Setting.Close();
	}
	if (RestoreChannelInfo.Channel>=0 && RestoreChannelInfo.szDriverName[0]!='\0'
			&& Setting.Open(m_szIniFileName,TEXT("LastChannel"),CSettings::OPEN_WRITE)) {
		TCHAR szDriverName[MAX_PATH],szName[32+MAX_PATH];

		::lstrcpy(szDriverName,::PathFindFileName(RestoreChannelInfo.szDriverName));
		*::PathFindExtension(szDriverName)='\0';
		::wsprintf(szName,TEXT("Space_%s"),szDriverName);
		Setting.Write(szName,RestoreChannelInfo.Space);
		::wsprintf(szName,TEXT("ChannelIndex_%s"),szDriverName);
		Setting.Write(szName,RestoreChannelInfo.Channel);
		::wsprintf(szName,TEXT("ServiceID_%s"),szDriverName);
		Setting.Write(szName,RestoreChannelInfo.ServiceID);
		Setting.Close();
	}
	StatusOptions.Save(m_szIniFileName);
	//ColorSchemeOptions.Save(m_szIniFileName);
	PluginOptions.Save(m_szIniFileName);
	return true;
}


bool CAppMain::ShowHelpContent(int ID)
{
	//return HtmlHelpClass.ShowContent(ID);
	return false;
}


bool CAppMain::StartRecord(LPCTSTR pszFileName,
						   const CRecordManager::TimeSpecInfo *pStartTime,
						   const CRecordManager::TimeSpecInfo *pStopTime)
{
	if (RecordManager.IsRecording())
		return false;
	RecordManager.SetFileName(pszFileName);
	RecordManager.SetStartTimeSpec(pStartTime);
	RecordManager.SetStopTimeSpec(pStopTime);
	RecordOptions.ApplyOptions(&RecordManager);
	if (CmdLineParser.m_fRecord1Seg)
		RecordManager.Set1SegOnly(true);
	else if (CmdLineParser.m_fRecordCurServiceOnly)
		RecordManager.SetCurServiceOnly(true);
	if (RecordManager.IsReserved()) {
		StatusView.UpdateItem(STATUS_ITEM_RECORD);
		return true;
	}

	MainWindow.OpenTuner();

	TCHAR szFileName[MAX_PATH];
	if (pszFileName==NULL) {
		LPCTSTR pszErrorMessage;

		if (!RecordOptions.GenerateFileName(szFileName,lengthof(szFileName),
											NULL,&pszErrorMessage)) {
			MainWindow.PopupErrorMessage(pszErrorMessage);
			return false;
		}
		RecordManager.SetFileName(szFileName);
		RecordManager.SetFileExistsOperation(RecordOptions.GetFileExistsOperation());
		if (!RecordManager.DoFileExistsOperation(MainWindow.GetVideoHostWindow(),szFileName))
			return false;
		pszFileName=szFileName;
	}
	if (RecordManager.Get1SegOnly())
		Set1SegService();
	CoreEngine.ResetErrorCount();
	if (!RecordManager.StartRecord(&CoreEngine.m_DtvEngine,pszFileName)) {
		TCHAR szMessage[64+MAX_PATH];

		::wsprintf(szMessage,TEXT("t@C \"%s\" J܂B"),pszFileName);
		Logger.AddLog(szMessage);
		if (RecordManager.GetLastErrorSystemMessage()!=NULL)
			Logger.AddLog(RecordManager.GetLastErrorSystemMessage());
		MainWindow.PopupErrorMessage(szMessage);
		return false;
	}
	ResidentManager.SetStatus(CResidentManager::STATUS_RECORDING,
							  CResidentManager::STATUS_RECORDING);
	Logger.AddLog(TEXT("^Jn %s"),pszFileName);
	StatusView.UpdateItem(STATUS_ITEM_RECORD);
	StatusView.UpdateItem(STATUS_ITEM_ERROR);
	MainMenu.EnableItem(CM_RECORDOPTION,false);
	MainMenu.EnableItem(CM_RECORDSTOPTIME,true);
	PluginList.SendRecordStatusChangeEvent();
	return true;
}


bool CAppMain::ModifyRecord(LPCTSTR pszFileName,
							const CRecordManager::TimeSpecInfo *pStartTime,
							const CRecordManager::TimeSpecInfo *pStopTime)
{
	RecordManager.SetFileName(pszFileName);
	RecordManager.SetStartTimeSpec(pStartTime);
	RecordManager.SetStopTimeSpec(pStopTime);
	StatusView.UpdateItem(STATUS_ITEM_RECORD);
	return true;
}


bool CAppMain::StartReservedRecord()
{
	TCHAR szFileName[MAX_PATH];

	/*
	if (!RecordManager.IsReserved())
		return false;
	*/
	if (RecordManager.GetFileName()!=NULL) {
		if (!RecordManager.DoFileExistsOperation(MainWindow.GetVideoHostWindow(),szFileName))
			return false;
	} else {
		LPCTSTR pszErrorMessage;

		if (!RecordOptions.GenerateFileName(szFileName,lengthof(szFileName),
													NULL,&pszErrorMessage)) {
			MainWindow.PopupMessage(pszErrorMessage);
			return false;
		}
		RecordManager.SetFileName(szFileName);
	}
	MainWindow.OpenTuner();
	if (RecordManager.Get1SegOnly())
		Set1SegService();
	CoreEngine.ResetErrorCount();
	if (!RecordManager.StartRecord(&CoreEngine.m_DtvEngine,szFileName)) {
		RecordManager.CancelReserve();
		TCHAR szMessage[64+MAX_PATH];

		::wsprintf(szMessage,TEXT("t@C \"%s\" J܂B"),szFileName);
		Logger.AddLog(szMessage);
		if (RecordManager.GetLastErrorSystemMessage()!=NULL)
			Logger.AddLog(RecordManager.GetLastErrorSystemMessage());
		MainWindow.PopupErrorMessage(szMessage);
		return false;
	}
	StatusView.UpdateItem(STATUS_ITEM_RECORD);
	StatusView.UpdateItem(STATUS_ITEM_ERROR);
	ResidentManager.SetStatus(CResidentManager::STATUS_RECORDING,
							  CResidentManager::STATUS_RECORDING);
	Logger.AddLog(TEXT("^Jn %s"),szFileName);
	MainMenu.EnableItem(CM_RECORDOPTION,false);
	MainMenu.EnableItem(CM_RECORDSTOPTIME,true);
	PluginList.SendRecordStatusChangeEvent();
	return true;
}


bool CAppMain::CancelReservedRecord()
{
	if (!RecordManager.CancelReserve())
		return false;
	StatusView.UpdateItem(STATUS_ITEM_RECORD);
	return true;
}


bool CAppMain::IsFirstExecute() const
{
	return m_fFirstExecute;
}


COLORREF CAppMain::GetColor(LPCTSTR pszText) const
{
	return ColorSchemeOptions.GetColor(pszText);
}


CCoreEngine *CAppMain::GetCoreEngine()
{
	return &CoreEngine;
}


const CCoreEngine *CAppMain::GetCoreEngine() const
{
	return &CoreEngine;
}


CMainWindow *CAppMain::GetMainWindow()
{
	return &MainWindow;
}


const CChannelManager *CAppMain::GetChannelManager() const
{
	return &ChannelManager;
}


const CRecordManager *CAppMain::GetRecordManager() const
{
	return &RecordManager;
}


CAppMain &GetAppClass()
{
	return AppMain;
}




class CTunerStatusItem : public CStatusItem {
public:
	CTunerStatusItem();
	LPCTSTR GetName() const { return TEXT("`[i["); }
	void Draw(HDC hdc,const RECT *pRect);
	void DrawPreview(HDC hdc,const RECT *pRect);
	void OnLButtonDown(int x,int y);
	void OnRButtonDown(int x,int y);
};

CTunerStatusItem::CTunerStatusItem() : CStatusItem(STATUS_ITEM_TUNER,80)
{
}

void CTunerStatusItem::Draw(HDC hdc,const RECT *pRect)
{
	const CChannelInfo *pChInfo=ChannelManager.GetCurrentRealChannelInfo();
	LPCTSTR pszText;

	if (pChInfo!=NULL || ChannelManager.GetCurrentSpace()>=0) {
		pszText=
			ChannelManager.GetDriverTuningSpaceList()->GetTuningSpaceName(
				pChInfo!=NULL?pChInfo->GetSpace():ChannelManager.GetCurrentSpace());
		if (pszText==NULL)
			pszText=TEXT("<`[i[>");
	} else if (ChannelManager.GetCurrentSpace()==CChannelManager::SPACE_ALL) {
		pszText=TEXT("ׂ");
	} else {
		pszText=TEXT("<`[i[>");
	}
	DrawText(hdc,pRect,pszText);
}

void CTunerStatusItem::DrawPreview(HDC hdc,const RECT *pRect)
{
	DrawText(hdc,pRect,TEXT("nfW"));
}

void CTunerStatusItem::OnLButtonDown(int x,int y)
{
	POINT pt;

	GetMenuPos(&pt);
	MainMenu.PopupSubMenu(CMainMenu::SUBMENU_SPACE,TPM_RIGHTBUTTON,pt.x,pt.y,
						  MainWindow.GetHandle());
}

void CTunerStatusItem::OnRButtonDown(int x,int y)
{
	HMENU hmenu=AppMain.CreateTunerSelectMenu();
	POINT pt;

	GetMenuPos(&pt);
	::TrackPopupMenu(hmenu,TPM_RIGHTBUTTON,pt.x,pt.y,0,MainWindow.GetHandle(),NULL);
	::DestroyMenu(hmenu);
}


class CChannelStatusItem : public CStatusItem {
public:
	CChannelStatusItem();
	LPCTSTR GetName() const { return TEXT("`l"); }
	void Draw(HDC hdc,const RECT *pRect);
	void DrawPreview(HDC hdc,const RECT *pRect);
	void OnLButtonDown(int x,int y);
	void OnRButtonDown(int x,int y);
};

CChannelStatusItem::CChannelStatusItem() : CStatusItem(STATUS_ITEM_CHANNEL,96)
{
}

void CChannelStatusItem::Draw(HDC hdc,const RECT *pRect)
{
	const CChannelInfo *pInfo;
	TCHAR szText[4+MAX_CHANNEL_NAME];

	if (fWheelChannelChanging) {
		COLORREF crText,crBack;

		crText=::GetTextColor(hdc);
		crBack=::GetBkColor(hdc);
		::SetTextColor(hdc,MixColor(crText,crBack,128));
		pInfo=ChannelManager.GetChangingChannelInfo();
		::wsprintf(szText,TEXT("%d: %s"),pInfo->GetChannelNo(),pInfo->GetName());
	} else if ((pInfo=ChannelManager.GetCurrentChannelInfo())!=NULL) {
		::wsprintf(szText,TEXT("%d: %s"),pInfo->GetChannelNo(),pInfo->GetName());
	} else
		::lstrcpy(szText,TEXT("<`l>"));
	DrawText(hdc,pRect,szText);
}

void CChannelStatusItem::DrawPreview(HDC hdc,const RECT *pRect)
{
	DrawText(hdc,pRect,TEXT("AtJer"));
}

void CChannelStatusItem::OnLButtonDown(int x,int y)
{
	POINT pt;

	GetMenuPos(&pt);
	MainMenu.PopupSubMenu(CMainMenu::SUBMENU_CHANNEL,TPM_RIGHTBUTTON,
											pt.x,pt.y,MainWindow.GetHandle());
}

void CChannelStatusItem::OnRButtonDown(int x,int y)
{
	POINT pt;

	GetMenuPos(&pt);
	MainMenu.PopupSubMenu(CMainMenu::SUBMENU_SERVICE,TPM_RIGHTBUTTON,
											pt.x,pt.y,MainWindow.GetHandle());
}


class CRecordStatusItem : public CStatusItem {
	COLORREF m_CircleColor;

public:
	CRecordStatusItem();
// CStatusItem
	LPCTSTR GetName() const { return TEXT("^"); }
	void Draw(HDC hdc,const RECT *pRect);
	void DrawPreview(HDC hdc,const RECT *pRect);
	void OnLButtonDown(int x,int y);
	void OnRButtonDown(int x,int y);
// CRecordStatusItem
	void SetCircleColor(COLORREF Color);
};

CRecordStatusItem::CRecordStatusItem()
	: CStatusItem(STATUS_ITEM_RECORD,64)
	, m_CircleColor(RGB(223,63,0))
{
}

void CRecordStatusItem::Draw(HDC hdc,const RECT *pRect)
{
	int FontHeight=m_pStatus->GetFontHeight();
	RECT rc;
	TCHAR szText[32],*pszText;

	rc=*pRect;
	if (RecordManager.IsRecording()) {
		DWORD RecordSec=RecordManager.GetRecordTime()/1000;

		if (RecordManager.IsPaused()) {
			HBRUSH hbr=::CreateSolidBrush(::GetTextColor(hdc));
			RECT rc1;

			rc1.left=rc.left;
			rc1.top=rc.top+((rc.bottom-rc.top)-FontHeight)/2;
			rc1.right=rc1.left+FontHeight/2-1;
			rc1.bottom=rc1.top+FontHeight;
			::FillRect(hdc,&rc1,hbr);
			rc1.left=rc1.right+2;
			rc1.right=rc.left+FontHeight;
			::FillRect(hdc,&rc1,hbr);
			::DeleteObject(hbr);
		} else {
#if 0
			// Ellipseŏۂ`Ɖ
			HBRUSH hbr=::CreateSolidBrush(m_CircleColor);
			HBRUSH hbrOld;
			HPEN hpenOld;

			rc1.right=rc1.left+FontHeight;
			hbrOld=SelectBrush(hdc,hbr);
			hpenOld=SelectPen(hdc,::GetStockObject(NULL_PEN));
			::Ellipse(hdc,rc1.left,rc1.top,rc1.right,rc1.bottom);
			SelectPen(hdc,hpenOld);
			SelectBrush(hdc,hbrOld);
			::DeleteObject(hbr);
#else
			COLORREF OldTextColor=::SetTextColor(hdc,m_CircleColor);
			::DrawText(hdc,TEXT(""),-1,&rc,
					   DT_LEFT | DT_SINGLELINE | DT_VCENTER);
			::SetTextColor(hdc,OldTextColor);
#endif
		}
		rc.left+=FontHeight+4;
		::wsprintf(szText,TEXT("%d:%02d:%02d"),
					RecordSec/(60*60),(RecordSec/60)%60,RecordSec%60);
		pszText=szText;
	} else if (RecordManager.IsReserved()) {
		pszText=TEXT(" ^ҋ@");
	} else {
		pszText=TEXT(" <^>");
	}
	DrawText(hdc,&rc,pszText);
}

void CRecordStatusItem::DrawPreview(HDC hdc,const RECT *pRect)
{
	RECT rc=*pRect;
	COLORREF OldTextColor=::SetTextColor(hdc,m_CircleColor);
	::DrawText(hdc,TEXT(""),-1,&rc,DT_LEFT | DT_SINGLELINE | DT_VCENTER);
	::SetTextColor(hdc,OldTextColor);
	rc.left+=m_pStatus->GetFontHeight()+4;
	DrawText(hdc,&rc,TEXT("0:24:15"));
}

void CRecordStatusItem::OnLButtonDown(int x,int y)
{
	MainWindow.SendCommand(RecordManager.IsReserved()?CM_RECORDOPTION:CM_RECORD);
}

void CRecordStatusItem::OnRButtonDown(int x,int y)
{
	HMENU hmenu;
	POINT pt;

	hmenu=LoadMenu(hInst,MAKEINTRESOURCE(IDM_RECORD));
	EnableMenuItem(hmenu,CM_RECORD_PAUSE,
		MF_BYCOMMAND | (RecordManager.IsRecording()?MFS_ENABLED:MFS_GRAYED));
	CheckMenuItem(hmenu,CM_RECORD_PAUSE,
		MF_BYCOMMAND | (RecordManager.IsPaused()?MFS_CHECKED:MFS_UNCHECKED));
	EnableMenuItem(hmenu,CM_RECORDSTOPTIME,
		MF_BYCOMMAND | (RecordManager.IsRecording()?MFS_ENABLED:MFS_GRAYED));
	CheckMenuItem(hmenu,CM_EXITONRECORDINGSTOP,
		MF_BYCOMMAND | (MainWindow.GetExitOnRecordingStop()?MFS_CHECKED:MFS_UNCHECKED));
	GetMenuPos(&pt);
	TrackPopupMenu(GetSubMenu(hmenu,0),TPM_RIGHTBUTTON,pt.x,pt.y,0,
												MainWindow.GetHandle(),NULL);
	DestroyMenu(hmenu);
}

void CRecordStatusItem::SetCircleColor(COLORREF Color)
{
	if (m_CircleColor!=Color) {
		m_CircleColor=Color;
		Update();
	}
}


class CErrorStatusItem : public CStatusItem {
public:
	CErrorStatusItem();
	LPCTSTR GetName() const { return TEXT("G["); }
	void Draw(HDC hdc,const RECT *pRect);
	void DrawPreview(HDC hdc,const RECT *pRect);
	void OnLButtonDown(int x,int y);
};

CErrorStatusItem::CErrorStatusItem() : CStatusItem(STATUS_ITEM_ERROR,120)
{
}

void CErrorStatusItem::Draw(HDC hdc,const RECT *pRect)
{
	TCHAR szText[64];
	int Length;

	Length=wsprintf(szText,TEXT("D %u / E %u"),
					CoreEngine.GetContinuityErrorPacketCount(),
					CoreEngine.GetErrorPacketCount());
	if (CoreEngine.GetDescramble()
			&& CoreEngine.GetCardReaderType()!=CCardReader::READER_NONE)
		wsprintf(szText+Length,TEXT(" / S %u"),CoreEngine.GetScramblePacketCount());
	DrawText(hdc,pRect,szText);
}

void CErrorStatusItem::DrawPreview(HDC hdc,const RECT *pRect)
{
	DrawText(hdc,pRect,TEXT("D 0 / E 0 / S 127"));
}

void CErrorStatusItem::OnLButtonDown(int x,int y)
{
	CoreEngine.ResetErrorCount();
	Update();
}


class CSignalLevelStatusItem : public CStatusItem {
public:
	CSignalLevelStatusItem();
	LPCTSTR GetName() const { return TEXT("Mx"); }
	void Draw(HDC hdc,const RECT *pRect);
	void DrawPreview(HDC hdc,const RECT *pRect);
};

CSignalLevelStatusItem::CSignalLevelStatusItem() : CStatusItem(STATUS_ITEM_SIGNALLEVEL,120)
{
}

void CSignalLevelStatusItem::Draw(HDC hdc,const RECT *pRect)
{
	CCoreEngine::DriverType DriverType=CoreEngine.GetDriverType();
	float SignalLevel;
	int Level;
	TCHAR szText[64];

	SignalLevel=CoreEngine.GetSignalLevel();
	Level=(int)(SignalLevel*100.0f);
	/*if (DriverType==CCoreEngine::DRIVER_UDP
			|| (DriverType==CCoreEngine::DRIVER_HDUS && !fNewHDUSDriver)) {
		// rbg[ĝ
		wsprintf(szText,TEXT("%d.%02d Mbps"),Level/100,Level%100);
	} else */{
		// plƍ킹邽߂floatŌvZ
		int BitRate=(int)(CoreEngine.GetBitRateFloat()*100.0f);

		wsprintf(szText,TEXT("%d.%02d dB / %d.%02d Mbps"),
								Level/100,Level%100,BitRate/100,BitRate%100);
	}
	DrawText(hdc,pRect,szText);
}

void CSignalLevelStatusItem::DrawPreview(HDC hdc,const RECT *pRect)
{
	DrawText(hdc,pRect,TEXT("52.30 dB / 16.73 Mbps"));
}


class CBufferingStatusItem : public CStatusItem {
public:
	CBufferingStatusItem();
	LPCTSTR GetName() const { return TEXT("obt@O"); }
	void Draw(HDC hdc,const RECT *pRect);
	void DrawPreview(HDC hdc,const RECT *pRect);
};

CBufferingStatusItem::CBufferingStatusItem()
	: CStatusItem(STATUS_ITEM_BUFFERING,40)
{
}

void CBufferingStatusItem::Draw(HDC hdc,const RECT *pRect)
{
	TCHAR szText[32];

	::wsprintf(szText,TEXT("R %lu"),CoreEngine.GetStreamRemain());
	DrawText(hdc,pRect,szText);
}

void CBufferingStatusItem::DrawPreview(HDC hdc,const RECT *pRect)
{
	DrawText(hdc,pRect,TEXT("R 52"));
}


class CProgramInfoStatusItem : public CStatusItem {
	bool m_fNext;
public:
	CProgramInfoStatusItem();
	LPCTSTR GetName() const { return TEXT("ԑg"); }
	void Draw(HDC hdc,const RECT *pRect);
	void DrawPreview(HDC hdc,const RECT *pRect);
	void OnLButtonDown(int x,int y);
	void OnRButtonDown(int x,int y);
};

CProgramInfoStatusItem::CProgramInfoStatusItem() : CStatusItem(STATUS_ITEM_PROGRAMINFO,256)
{
	m_fNext=false;
}

void CProgramInfoStatusItem::Draw(HDC hdc,const RECT *pRect)
{
	TCHAR szText[256];
	SYSTEMTIME stStart,stEnd;

	if (CoreEngine.m_DtvEngine.GetEventTime(&stStart,&stEnd,m_fNext)) {
		int Length;

		Length=::wsprintf(szText,TEXT("%s%d:%02d`%d:%02d "),
						  m_fNext?TEXT(": "):TEXT(""),
						  stStart.wHour,stStart.wMinute,
						  stEnd.wHour,stEnd.wMinute);
		if (!CoreEngine.m_DtvEngine.GetEventName(&szText[Length],lengthof(szText)-Length,m_fNext))
			szText[Length]='\0';
		DrawText(hdc,pRect,szText);
	}
}

void CProgramInfoStatusItem::DrawPreview(HDC hdc,const RECT *pRect)
{
	DrawText(hdc,pRect,TEXT("1:00`1:30 ̃j[X"));
}

void CProgramInfoStatusItem::OnLButtonDown(int x,int y)
{
	m_fNext=!m_fNext;
	Update();
}

void CProgramInfoStatusItem::OnRButtonDown(int x,int y)
{
}


class CNetworkStatusItem : public CStatusItem {
public:
	CNetworkStatusItem();
	LPCTSTR GetName() const { return TEXT("lbg[N"); }
	void Draw(HDC hdc,const RECT *pRect);
	void DrawPreview(HDC hdc,const RECT *pRect);
	void OnLButtonDown(int x,int y);
	void OnRButtonDown(int x,int y);
};

CNetworkStatusItem::CNetworkStatusItem()
	: CStatusItem(STATUS_ITEM_NETWORK,64)
{
}

void CNetworkStatusItem::Draw(HDC hdc,const RECT *pRect)
{
	TCHAR szText[32];

	if (UdpOptions.IsNetworkSending()) {
		::wsprintf(szText,TEXT("%s %u"),
			UdpOptions.GetProtocol()==CUdpOptions::PROTOCOL_UDP?TEXT("UDP"):TEXT("TCP"),
			UdpOptions.GetSendingPort());
	} else {
		::lstrcpy(szText, TEXT("<Network>"));
	}
	DrawText(hdc,pRect,szText);
}

void CNetworkStatusItem::DrawPreview(HDC hdc,const RECT *pRect)
{
	DrawText(hdc,pRect,TEXT("UDP 1234"));
}


void CNetworkStatusItem::OnLButtonDown(int x,int y)
{
	int Command;

	switch (UdpOptions.GetStatusMode()) {
	case CUdpOptions::STATUS_SENDANDVIEW:
		if (UdpOptions.IsNetworkSending())
			Command=CM_UDPSEND;
		else
			Command=CM_UDPVIEW;
		break;
	case CUdpOptions::STATUS_SEND:
		Command=CM_UDPSEND;
		break;
	case CUdpOptions::STATUS_NETWORKVIEW:
		Command=CM_NETWORKVIEW;
		break;
	case CUdpOptions::STATUS_CHASINGVIEW:
		Command=CM_CHASINGVIEW;
		break;
	default:
		return;
	}
	MainWindow.PostCommand(Command);
}


void CNetworkStatusItem::OnRButtonDown(int x,int y)
{
	HMENU hmenu;
	POINT pt;

	hmenu=::LoadMenu(hInst,MAKEINTRESOURCE(IDM_NETWORK));
	::CheckMenuItem(hmenu,CM_UDPSEND,
		MF_BYCOMMAND | (UdpOptions.IsNetworkSending()?MFS_CHECKED:MFS_UNCHECKED));
	GetMenuPos(&pt);
	::TrackPopupMenu(GetSubMenu(hmenu,0),TPM_RIGHTBUTTON,pt.x,pt.y,0,
												MainWindow.GetHandle(),NULL);
	::DestroyMenu(hmenu);
}




static bool ColorSchemeApplyProc(const CColorScheme *pColorScheme)
{
	StatusView.SetColor(
		pColorScheme->GetColor(CColorScheme::COLOR_STATUSBACK1),
		pColorScheme->GetColor(CColorScheme::COLOR_STATUSBACK2),
		pColorScheme->GetColor(CColorScheme::COLOR_STATUSTEXT),
		pColorScheme->GetColor(CColorScheme::COLOR_STATUSHIGHLIGHTBACK1),
		pColorScheme->GetColor(CColorScheme::COLOR_STATUSHIGHLIGHTBACK2),
		pColorScheme->GetColor(CColorScheme::COLOR_STATUSHIGHLIGHTTEXT));
	CRecordStatusItem *pRecordStatus=dynamic_cast<CRecordStatusItem*>(StatusView.GetItemByID(STATUS_ITEM_RECORD));
	if (pRecordStatus!=NULL)
		pRecordStatus->SetCircleColor(pColorScheme->GetColor(CColorScheme::COLOR_STATUSRECORDINGCIRCLE));
	//PluginList.SendColorChangeEvent();
	return true;
}




class CGeneralOptions : public COptions {
	static CGeneralOptions *GetThis(HWND hDlg);
public:
	static INT_PTR CALLBACK DlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam);
};

CGeneralOptions *CGeneralOptions::GetThis(HWND hDlg)
{
	return static_cast<CGeneralOptions*>(::GetProp(hDlg,TEXT("This")));
}

INT_PTR CALLBACK CGeneralOptions::DlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		OnInitDialog(hDlg,lParam);
		{
			TCHAR szDirectory[MAX_PATH];

			::SendDlgItemMessage(hDlg,IDC_OPTIONS_DRIVER,CB_LIMITTEXT,MAX_PATH-1,0);
			AppMain.GetAppDirectory(szDirectory);
			DriverManager.Find(szDirectory);
			AppMain.UpdateDriverMenu();
			for (int i=0;i<DriverManager.NumDrivers();i++) {
				const CDriverInfo *pDriverInfo=DriverManager.GetDriverInfo(i);

				::SendDlgItemMessage(hDlg,IDC_OPTIONS_DRIVER,CB_ADDSTRING,
					0,reinterpret_cast<LPARAM>(pDriverInfo->GetFileName()));
			}
			::SetDlgItemText(hDlg,IDC_OPTIONS_DRIVER,szDriverFileName);
		}
		{
			static const LPCTSTR pszCardReaderList[] = {
				TEXT("Ȃ(XNuȂ)"),
				TEXT("X}[gJ[h[_"),
				TEXT("HDUSJ[h[_")
			};

			for (int i=0;i<lengthof(pszCardReaderList);i++)
				SendDlgItemMessage(hDlg,IDC_OPTIONS_CARDREADER,CB_ADDSTRING,0,(LPARAM)pszCardReaderList[i]);
			SendDlgItemMessage(hDlg,IDC_OPTIONS_CARDREADER,CB_SETCURSEL,(WPARAM)CoreEngine.GetCardReaderType(),0);
		}
		CheckDlgButton(hDlg,IDC_OPTIONS_RESIDENT,
			ResidentManager.GetResident()?BST_CHECKED:BST_UNCHECKED);
		CheckDlgButton(hDlg,IDC_OPTIONS_RESTORECHANNEL,
									fRestoreChannel?BST_CHECKED:BST_UNCHECKED);
		CheckDlgButton(hDlg,IDC_OPTIONS_SNAPATWINDOWEDGE,
			fSnapAtWindowEdge?BST_CHECKED:BST_UNCHECKED);
		if (CTsDescrambler::IsSSE2Available())
			DlgCheckBox_Check(hDlg,IDC_OPTIONS_DESCRAMBLEUSESSE2,fDescrambleUseSSE2);
		else
			EnableDlgItems(hDlg,IDC_OPTIONS_DESCRAMBLEUSESSE2,
								IDC_OPTIONS_DESCRAMBLEBENCHMARK,false);
		DlgCheckBox_Check(hDlg,IDC_OPTIONS_DESCRAMBLECURSERVICEONLY,fDescrambleCurServiceOnly);
		DlgCheckBox_Check(hDlg,IDC_OPTIONS_ENABLEEMMPROCESS,fEnableEmmProcess);
		DlgCheckBox_Check(hDlg,IDC_OPTIONS_NOTASKBAR,fNoTaskBar);
		DlgCheckBox_Check(hDlg,IDC_OPTIONS_MINIMIZETOTRAY,ResidentManager.GetMinimizeToTray());
		{
			static const LPCTSTR pszInitialPosList[] = {
				TEXT("ŌɏIʒu"),
				TEXT(""),
				TEXT("E"),
				TEXT(""),
				TEXT("E"),
			};

			for (int i=0;i<lengthof(pszInitialPosList);i++)
				SendDlgItemMessage(hDlg,IDC_OPTIONS_INITIALPOS,CB_ADDSTRING,0,(LPARAM)pszInitialPosList[i]);
			SendDlgItemMessage(hDlg,IDC_OPTIONS_INITIALPOS,CB_SETCURSEL,InitialPos,0);
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_OPTIONS_DRIVER_BROWSE:
			{
				OPENFILENAME ofn;
				TCHAR szFileName[MAX_PATH],szInitDir[MAX_PATH];
				CFilePath FilePath;

				::GetDlgItemText(hDlg,IDC_OPTIONS_DRIVER,szFileName,lengthof(szFileName));
				FilePath.SetPath(szFileName);
				if (FilePath.GetDirectory(szInitDir)) {
					::lstrcpy(szFileName,FilePath.GetFileName());
				} else {
					GetAppClass().GetAppDirectory(szInitDir);
				}
				InitOpenFileName(&ofn);
				ofn.hwndOwner=hDlg;
				ofn.lpstrFilter=
					TEXT("BonDriver(BonDriver*.dll)\0BonDriver*.dll\0")
					TEXT("ׂẴt@C\0*.*\0");
				ofn.lpstrFile=szFileName;
				ofn.nMaxFile=lengthof(szFileName);
				ofn.lpstrInitialDir=szInitDir;
				ofn.lpstrTitle=TEXT("BonDriveȓI");
				ofn.Flags=OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_EXPLORER;
				if (::GetOpenFileName(&ofn)) {
					::SetDlgItemText(hDlg,IDC_OPTIONS_DRIVER,szFileName);
				}
			}
			return TRUE;
		}
		return TRUE;

	case WM_NOTIFY:
		switch (((LPNMHDR)lParam)->code) {
		case PSN_APPLY:
			{
				CGeneralOptions *pThis=GetThis(hDlg);
				TCHAR szDriver[MAX_PATH];

				GetDlgItemText(hDlg,IDC_OPTIONS_DRIVER,szDriver,lengthof(szDriver));
				if (szDriver[0]!='\0'
						&& lstrcmpi(szDriver,szDriverFileName)!=0) {
					::lstrcpy(szDriverFileName,szDriver);
					pThis->m_UpdateFlags|=UPDATE_DRIVER;
				}
			}
			if (!CoreEngine.SetCardReaderType((CCardReader::ReaderType)
					SendDlgItemMessage(hDlg,IDC_OPTIONS_CARDREADER,CB_GETCURSEL,0,0))) {
				Logger.AddLog(CoreEngine.GetLastErrorText());
				MainWindow.PopupErrorMessage(CoreEngine.GetLastErrorText());
				CoreEngine.SetCardReaderType(CCardReader::READER_NONE);
			}
			ResidentManager.SetResident(
				IsDlgButtonChecked(hDlg,IDC_OPTIONS_RESIDENT)==BST_CHECKED);
			fRestoreChannel=IsDlgButtonChecked(hDlg,
									IDC_OPTIONS_RESTORECHANNEL)==BST_CHECKED;
			fSnapAtWindowEdge=IsDlgButtonChecked(hDlg,
									IDC_OPTIONS_SNAPATWINDOWEDGE)==BST_CHECKED;
			fDescrambleUseSSE2=DlgCheckBox_IsChecked(hDlg,IDC_OPTIONS_DESCRAMBLEUSESSE2);
			{
				bool fCurOnly=DlgCheckBox_IsChecked(hDlg,IDC_OPTIONS_DESCRAMBLECURSERVICEONLY)==BST_CHECKED;
				if (fCurOnly!=fDescrambleCurServiceOnly) {
					fDescrambleCurServiceOnly=fCurOnly;
					if (!RecordManager.IsRecording())
						CoreEngine.m_DtvEngine.SetDescrambleCurServiceOnly(fDescrambleCurServiceOnly);
				}
			}
			{
				bool fEmm=DlgCheckBox_IsChecked(hDlg,IDC_OPTIONS_ENABLEEMMPROCESS);
				if (fEmm!=fEnableEmmProcess) {
					fEnableEmmProcess=fEmm;
					CoreEngine.m_DtvEngine.m_TsDescrambler.EnableEmmProcess(fEnableEmmProcess);
				}
			}
			{
				bool f=DlgCheckBox_IsChecked(hDlg,IDC_OPTIONS_NOTASKBAR);
				if (f!=fNoTaskBar) {
					fNoTaskBar=f;
					// x\ɂȂƃ^XNo[Ȃ
					if (f)
						MainWindow.SetVisible(false);
					MainWindow.SetExStyle(MainWindow.GetExStyle()^WS_EX_TOOLWINDOW);
					if (f)
						MainWindow.SetVisible(true);
					ResidentManager.SetAlwaysTray(fNoTaskBar);
				}
			}
			ResidentManager.SetMinimizeToTray(DlgCheckBox_IsChecked(hDlg,IDC_OPTIONS_MINIMIZETOTRAY));
			InitialPos=(int)DlgComboBox_GetCurSel(hDlg,IDC_OPTIONS_INITIALPOS);
			return TRUE;
		}
		break;

	case WM_DESTROY:
		{
			CGeneralOptions *pThis=GetThis(hDlg);

			pThis->OnDestroy();
		}
		return TRUE;
	}
	return FALSE;
}


class COperationOptions : public COptions {
public:
	static INT_PTR CALLBACK DlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam);
};

INT_PTR CALLBACK COperationOptions::DlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		{
			CheckDlgButton(hDlg,IDC_OPTIONS_WHEELCHANNELREVERSE,
							fWheelChannelReverse?BST_CHECKED:BST_UNCHECKED);
			SetDlgItemInt(hDlg,IDC_OPTIONS_WHEELCHANNELDELAY,
													WheelChannelDelay,FALSE);
			CheckDlgButton(hDlg,IDC_OPTIONS_CHANGECH_FUNCTION,
						fFunctionKeyChangeChannel?BST_CHECKED:BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_OPTIONS_CHANGECH_DIGIT,
						fDigitKeyChangeChannel?BST_CHECKED:BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_OPTIONS_CHANGECH_NUMPAD,
						fNumPadChangeChannel?BST_CHECKED:BST_UNCHECKED);
		}
		return TRUE;

	case WM_NOTIFY:
		switch (((LPNMHDR)lParam)->code) {
		case PSN_APPLY:
			{
				fWheelChannelReverse=IsDlgButtonChecked(hDlg,
								IDC_OPTIONS_WHEELCHANNELREVERSE)==BST_CHECKED;
				WheelChannelDelay=GetDlgItemInt(hDlg,
									IDC_OPTIONS_WHEELCHANNELDELAY,NULL,FALSE);
				fFunctionKeyChangeChannel=IsDlgButtonChecked(hDlg,
								IDC_OPTIONS_CHANGECH_FUNCTION)==BST_CHECKED;
				fDigitKeyChangeChannel=IsDlgButtonChecked(hDlg,
									IDC_OPTIONS_CHANGECH_DIGIT)==BST_CHECKED;
				fNumPadChangeChannel=IsDlgButtonChecked(hDlg,
									IDC_OPTIONS_CHANGECH_NUMPAD)==BST_CHECKED;
			}
			break;
		}
		break;
	}
	return FALSE;
}


static CGeneralOptions GeneralOptions;
static COperationOptions OperationOptions;


class COptionDialog {
public:
	enum {
		PAGE_GENERAL,
		PAGE_STATUS,
		PAGE_OPERATION,
		PAGE_RECORD,
		//PAGE_EPG,
		PAGE_UDP,
		PAGE_PLUGIN,
		PAGE_LOG,
		NUM_PAGES
	};
	COptionDialog();
	~COptionDialog();
	bool ShowDialog(HWND hwndOwner,int StartPage=-1);
private:
	struct PageInfo {
		LPCTSTR pszTitle;
		LPCTSTR pszTemplate;
		DLGPROC pfnDlgProc;
		COptions *pOptions;
		COLORREF crTitleColor;
	};
	static const PageInfo m_PageList[NUM_PAGES];
	int m_CurrentPage;
	int m_StartPage;
	HWND m_hDlg;
	HWND m_hDlgList[NUM_PAGES];
	HFONT m_hfontTitle;
	DWORD m_UpdateFlags;
	void CreatePage(int Page);
	static COptionDialog *GetThis(HWND hDlg);
	static INT_PTR CALLBACK DlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam);
};


const COptionDialog::PageInfo COptionDialog::m_PageList[] = {
	{TEXT(""),					MAKEINTRESOURCE(IDD_OPTIONS_GENERAL),
		CGeneralOptions::DlgProc,		&GeneralOptions,	RGB(128,0,0)},
	{TEXT("Xe[^Xo["),		MAKEINTRESOURCE(IDD_OPTIONS_STATUS),
		CStatusOptions::DlgProc,		&StatusOptions,		RGB(0,0,128)},
	{TEXT(""),					MAKEINTRESOURCE(IDD_OPTIONS_OPERATION),
		COperationOptions::DlgProc,		&OperationOptions,	RGB(128,128,0)},
	{TEXT("^"),					MAKEINTRESOURCE(IDD_OPTIONS_RECORD),
		CRecordOptions::DlgProc,		&RecordOptions,		RGB(128,0,160)},
	/*
	{TEXT("EPG"),					MAKEINTRESOURCE(IDD_OPTIONS_EPG),
		CEpgOptions::DlgProc,			&EpgOptions,		RGB(128,0,255)},
	*/
	{TEXT("/lbg[N"),		MAKEINTRESOURCE(IDD_OPTIONS_UDP),
		CUdpOptions::DlgProc,			&UdpOptions,		RGB(0,255,128)},
	{TEXT("vOC"),			MAKEINTRESOURCE(IDD_OPTIONS_PLUGIN),
		CPluginOptions::DlgProc,	&PluginOptions,			RGB(255,0,160)},
	{TEXT("O"),					MAKEINTRESOURCE(IDD_OPTIONS_LOG),
		CLogger::DlgProc,				&Logger,			RGB(255,0,128)},
};


COptionDialog::COptionDialog()
{
	m_CurrentPage=0;
	m_hfontTitle=NULL;
}


COptionDialog::~COptionDialog()
{
	if (m_hfontTitle)
		::DeleteObject(m_hfontTitle);
}


void COptionDialog::CreatePage(int Page)
{
	if (m_hDlgList[Page]==NULL) {
		RECT rc;
		POINT pt;

		m_hDlgList[Page]=::CreateDialogParam(hInst,
			m_PageList[Page].pszTemplate,m_hDlg,m_PageList[Page].pfnDlgProc,
			reinterpret_cast<LPARAM>(m_PageList[Page].pOptions));
		::GetWindowRect(::GetDlgItem(m_hDlg,IDC_OPTIONS_PAGEPLACE),&rc);
		pt.x=rc.left;
		pt.y=rc.top;
		::ScreenToClient(m_hDlg,&pt);
		::SetWindowPos(m_hDlgList[Page],NULL,pt.x,pt.y,0,0,
													SWP_NOZORDER | SWP_NOSIZE);
	}
}


COptionDialog *COptionDialog::GetThis(HWND hDlg)
{
	return static_cast<COptionDialog*>(::GetProp(hDlg,TEXT("This")));
}


INT_PTR CALLBACK COptionDialog::DlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		{
			COptionDialog *pThis=reinterpret_cast<COptionDialog*>(lParam);

			::SetProp(hDlg,TEXT("This"),pThis);
			pThis->m_hDlg=hDlg;
			for (int i=0;i<NUM_PAGES;i++) {
				pThis->m_hDlgList[i]=NULL;
				::SendDlgItemMessage(hDlg,IDC_OPTIONS_LIST,LB_ADDSTRING,0,
					reinterpret_cast<LPARAM>(pThis->m_PageList[i].pszTitle));
			}
			if (pThis->m_StartPage>=0 && pThis->m_StartPage<NUM_PAGES)
				pThis->m_CurrentPage=pThis->m_StartPage;
			pThis->CreatePage(pThis->m_CurrentPage);
			::ShowWindow(pThis->m_hDlgList[pThis->m_CurrentPage],SW_SHOW);
			::SendDlgItemMessage(hDlg,IDC_OPTIONS_LIST,LB_SETCURSEL,pThis->m_CurrentPage,0);
			if (pThis->m_hfontTitle==NULL) {
				HFONT hfont;
				LOGFONT lf;

				hfont=(HFONT)::SendMessage(hDlg,WM_GETFONT,0,0);
				::GetObject(hfont,sizeof(LOGFONT),&lf);
				lf.lfWeight=FW_BOLD;
				pThis->m_hfontTitle=::CreateFontIndirect(&lf);
			}
		}
		return TRUE;

	case WM_DRAWITEM:
		{
			COptionDialog *pThis=GetThis(hDlg);
			LPDRAWITEMSTRUCT pdis=(LPDRAWITEMSTRUCT)lParam;
			HFONT hfontOld;
			COLORREF crOldText;
			int OldBkMode;
			RECT rc;

			DrawUtil::FillGradient(pdis->hDC,&pdis->rcItem,
				RGB(0,0,0),pThis->m_PageList[pThis->m_CurrentPage].crTitleColor);
			hfontOld=SelectFont(pdis->hDC,pThis->m_hfontTitle);
			crOldText=::SetTextColor(pdis->hDC,RGB(255,255,255));
			OldBkMode=::SetBkMode(pdis->hDC,TRANSPARENT);
			rc.left=pdis->rcItem.left+2;
			rc.top=pdis->rcItem.top;
			rc.right=pdis->rcItem.right-2;
			rc.bottom=pdis->rcItem.bottom;
			::DrawText(pdis->hDC,pThis->m_PageList[pThis->m_CurrentPage].pszTitle,-1,
					&rc,DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX);
			SelectFont(pdis->hDC,hfontOld);
			::SetTextColor(pdis->hDC,crOldText);
			::SetBkMode(pdis->hDC,OldBkMode);
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_OPTIONS_LIST:
			if (HIWORD(wParam)==LBN_SELCHANGE) {
				COptionDialog *pThis=GetThis(hDlg);
				int NewPage=(int)SendDlgItemMessage(hDlg,IDC_OPTIONS_LIST,LB_GETCURSEL,0,0);

				if (pThis->m_hDlgList[NewPage]==NULL) {
					HCURSOR hcurOld;

					hcurOld=::SetCursor(LoadCursor(NULL,IDC_WAIT));
					pThis->CreatePage(NewPage);
					::SetCursor(hcurOld);
				}
				::ShowWindow(pThis->m_hDlgList[pThis->m_CurrentPage],SW_HIDE);
				::ShowWindow(pThis->m_hDlgList[NewPage],SW_SHOW);
				pThis->m_CurrentPage=NewPage;
				::InvalidateRect(::GetDlgItem(hDlg,IDC_OPTIONS_TITLE),NULL,TRUE);
			}
			return TRUE;

		/*
		case IDC_OPTIONS_HELP:
			{
				COptionDialog *pThis=GetThis(hDlg);

				HtmlHelpClass.ShowContent(HELP_ID_OPTIONS_FIRST+pThis->m_CurrentPage);
			}
			return TRUE;
		*/

		case IDOK:
		case IDCANCEL:
			{
				COptionDialog *pThis=GetThis(hDlg);
				HCURSOR hcurOld;
				NMHDR nmh;
				int i;

				hcurOld=::SetCursor(::LoadCursor(NULL,IDC_WAIT));
				nmh.code=LOWORD(wParam)==IDOK?PSN_APPLY:PSN_RESET;
				pThis->m_UpdateFlags=0;
				for (i=0;i<NUM_PAGES;i++) {
					if (pThis->m_hDlgList[i]!=NULL) {
						::SendMessage(pThis->m_hDlgList[i],WM_NOTIFY,0,(LPARAM)&nmh);
						pThis->m_UpdateFlags|=m_PageList[i].pOptions->GetUpdateFlags();
					}
				}
				::SetCursor(hcurOld);
				::EndDialog(hDlg,LOWORD(wParam));
			}
			return TRUE;
		}
		return TRUE;

	case WM_DESTROY:
		::RemoveProp(hDlg,TEXT("This"));
		return TRUE;
	}
	return FALSE;
}


bool COptionDialog::ShowDialog(HWND hwndOwner,int StartPage)
{
	m_StartPage=StartPage;
	if (::DialogBoxParam(hInst,MAKEINTRESOURCE(IDD_OPTIONS),hwndOwner,
						 DlgProc,reinterpret_cast<LPARAM>(this))!=IDOK)
		return false;
	MainWindow.Update();
	AppMain.SaveSettings();
	if (m_UpdateFlags&COptions::UPDATE_DRIVER) {
		if (AppMain.SetDriver(szDriverFileName))
			m_UpdateFlags|=COptions::UPDATE_PREVIEW;
	}
	StatusView.UpdateItem(STATUS_ITEM_NETWORK);
	return true;
}


static COptionDialog OptionDialog;




class CServiceUpdateInfo {
public:
	struct ServiceInfo {
		WORD ServiceID;
		TCHAR szServiceName[256];
		BYTE VideoStreamType;
	};
	ServiceInfo *m_pServiceList;
	int m_NumServices;
	int m_CurService;
	WORD m_NetworkID;
	WORD m_TransportStreamID;
	CServiceUpdateInfo(CDtvEngine *pEngine,CProgManager *pProgManager);
	~CServiceUpdateInfo();
};

CServiceUpdateInfo::CServiceUpdateInfo(CDtvEngine *pEngine,CProgManager *pProgManager)
{
	m_NumServices=pProgManager->GetServiceNum();
	if (m_NumServices>0) {
		m_pServiceList=new ServiceInfo[m_NumServices];
		for (int i=0;i<m_NumServices;i++) {
			if (!pProgManager->GetServiceID(&m_pServiceList[i].ServiceID,i))
				m_pServiceList[i].ServiceID=0;
			if (!pProgManager->GetServiceName(m_pServiceList[i].szServiceName,i))
				m_pServiceList[i].szServiceName[0]='\0';
			if (!pProgManager->GetVideoStreamType(&m_pServiceList[i].VideoStreamType,i))
				m_pServiceList[i].VideoStreamType=0;
		}
	} else {
		m_pServiceList=NULL;
	}
	m_CurService=pEngine->GetService();
	if (m_CurService>=m_NumServices)
		m_CurService=-1;
	m_NetworkID=pProgManager->GetNetworkID();
	m_TransportStreamID=pProgManager->GetTransportStreamID();
}

CServiceUpdateInfo::~CServiceUpdateInfo()
{
	delete [] m_pServiceList;
}


class MyDtvEngineHandler : public CDtvEngineHandler
{
	const DWORD OnDtvEngineEvent(CDtvEngine *pEngine,const DWORD dwEventID,
																PVOID pParam);
};

const DWORD MyDtvEngineHandler::OnDtvEngineEvent(CDtvEngine *pEngine,
											const DWORD dwEventID,PVOID pParam)
{
	switch (dwEventID) {
	case CDtvEngine::EID_SERVICE_INFO_UPDATED:
	case CDtvEngine::EID_SERVICE_LIST_UPDATED:
		{
			CProgManager *pProgManager=static_cast<CProgManager*>(pParam);

			MainWindow.PostMessage(WM_APP_SERVICEUPDATE,0,
				reinterpret_cast<LPARAM>(new CServiceUpdateInfo(pEngine,pProgManager)));
		}
		break;
	case CDtvEngine::EID_FILE_WRITE_ERROR:
		MainWindow.PostMessage(WM_APP_FILEWRITEERROR,0,0);
		break;
	}
	return 0;
}


static MyDtvEngineHandler DtvEngineHandler;




bool CMainWindow::Initialize()
{
	WNDCLASS wc;

	wc.style=CS_DBLCLKS;
	wc.lpfnWndProc=WndProc;
	wc.cbClsExtra=0;
	wc.cbWndExtra=0;
	wc.hInstance=hInst;
	wc.hIcon=LoadIcon(hInst,MAKEINTRESOURCE(IDI_ICON));
	wc.hCursor=LoadCursor(NULL,IDC_ARROW);
	wc.hbrBackground=NULL;//(HBRUSH)(COLOR_3DFACE+1);
	wc.lpszMenuName=NULL;
	wc.lpszClassName=MAIN_WINDOW_CLASS;
	return RegisterClass(&wc)!=0;
}


CMainWindow::CMainWindow()
{
	// KɃftHgʒuݒ
	m_WindowPosition.Width=960;
	m_WindowPosition.Height=16;
	m_WindowPosition.Left=(::GetSystemMetrics(SM_CXSCREEN)-m_WindowPosition.Width)/2;
	m_WindowPosition.Top=(::GetSystemMetrics(SM_CYSCREEN)-m_WindowPosition.Height)/2;
	m_fAlwaysOnTop=false;
	m_fStandby=false;
	m_fStandbyInit=false;
	m_fMinimizeInit=false;
	m_fSrcFilterReleased=false;
	m_fExitOnRecordingStop=false;
	m_fClosing=false;

	NONCLIENTMETRICS ncm;
#if WINVER<0x0600
	ncm.cbSize=sizeof(ncm);
#else
	ncm.cbSize=offsetof(NONCLIENTMETRICS,iPaddedBorderWidth);
#endif
	::SystemParametersInfo(SPI_GETNONCLIENTMETRICS,sizeof(ncm),&ncm,0);
	m_hfontCaption=::CreateFontIndirect(&ncm.lfCaptionFont);
}


CMainWindow::~CMainWindow()
{
	::DeleteObject(m_hfontCaption);
}


bool CMainWindow::Create(HWND hwndParent,DWORD Style,DWORD ExStyle,int ID)
{
	if (m_fAlwaysOnTop)
		ExStyle|=WS_EX_TOPMOST;
	if (!CreateBasicWindow(NULL,Style,ExStyle,ID,MAIN_WINDOW_CLASS,TITLE_TEXT,hInst))
		return false;
	return true;
}


bool CMainWindow::Show(int CmdShow)
{
	return ::ShowWindow(m_hwnd,CmdShow)!=FALSE;
}


HWND CMainWindow::GetVideoHostWindow() const
{
	if (m_fStandby)
		return NULL;
	return m_hwnd;
}


int CMainWindow::ShowMessage(LPCTSTR pszText,LPCTSTR pszCaption,UINT Type) const
{
	return ::MessageBox(GetVideoHostWindow(),pszText,pszCaption,Type);
}


bool CMainWindow::PopupMessage(LPCTSTR pszText,LPCTSTR pszTitle,int Icon,DWORD TimeOut)
{
	RECT rc;
	POINT pt;

	if (!GetVisible() || ::IsIconic(m_hwnd)) {
		if (!GetVisible() || ResidentManager.GetMinimizeToTray())
			return ResidentManager.ShowMessage(pszText,pszTitle,Icon,TimeOut);
#if 1
		::SystemParametersInfo(SPI_GETWORKAREA,0,&rc,0);
		pt.x=rc.right-256;
		pt.y=rc.bottom;
#else
		APPBARDATA abd;

		::ZeroMemory(&abd,sizeof(abd));
		abd.cbSize=sizeof(abd);
		::SHAppBarMessage(ABM_GETTASKBARPOS,&abd);
		pt.x=abd.rc.right-256;
		pt.y=abd.rc.top;
#endif
	} else {
		::GetClientRect(m_hwnd,&rc);
		pt.x=0;
		pt.y=rc.bottom;
		::ClientToScreen(m_hwnd,&pt);
	}
	if (!ToolTip.Show(pszText,pszTitle,pt.x,pt.y,Icon))
		return false;
	if (TimeOut>0)
		::SetTimer(m_hwnd,TIMER_ID_INFOTIP,TimeOut,NULL);
	return true;
}


struct AdjustPosInfo {
	HWND hwnd;
	RECT rc;
	bool fMoved;
};

BOOL CALLBACK CMainWindow::AdjustPosProc(HWND hwnd,LPARAM lParam)
{
	TCHAR szName[64];

	if (::GetClassName(hwnd,szName,lengthof(szName))
			&& ::lstrcmpi(szName,MAIN_WINDOW_CLASS)==0) {
		AdjustPosInfo *pInfo=(AdjustPosInfo*)lParam;

		if (hwnd!=pInfo->hwnd) {
			RECT rc;

			::GetWindowRect(hwnd,&rc);
			if (rc.bottom>pInfo->rc.top && rc.top<pInfo->rc.bottom) {
				::OffsetRect(&pInfo->rc,0,
					InitialPos==INITIALPOS_LOWERLEFT || InitialPos==INITIALPOS_LOWERRIGHT?
							rc.top-pInfo->rc.bottom:rc.bottom-pInfo->rc.top);
				pInfo->fMoved=true;
			}
		}
	}
	return TRUE;
}

void CMainWindow::AdjustPos()
{
	if (InitialPos!=INITIALPOS_LAST) {
		HMONITOR hMonitor=::MonitorFromWindow(m_hwnd,MONITOR_DEFAULTTOPRIMARY);
		MONITORINFO mi;
		RECT rc;
		int Width=GetWidth(),Height=GetHeight();

		mi.cbSize=sizeof(mi);
		::GetMonitorInfo(hMonitor,&mi);
		switch (InitialPos) {
		case INITIALPOS_LOWERLEFT:
			rc.left=mi.rcWork.left;
			rc.top=mi.rcWork.bottom-Height;
			rc.right=rc.left+Width;
			rc.bottom=mi.rcWork.bottom;
			break;
		case INITIALPOS_LOWERRIGHT:
			rc.left=mi.rcWork.right-Width;
			rc.top=mi.rcWork.bottom-Height;
			rc.right=mi.rcWork.right;
			rc.bottom=mi.rcWork.bottom;
			break;
		case INITIALPOS_UPPERLEFT:
			rc.left=mi.rcWork.left;
			rc.top=mi.rcWork.top;
			rc.right=rc.left+Width;
			rc.bottom=mi.rcWork.top+Height;
			break;
		case INITIALPOS_UPPERRIGHT:
			rc.left=mi.rcWork.right-Width;
			rc.top=mi.rcWork.top;
			rc.right=mi.rcWork.right;
			rc.bottom=mi.rcWork.top+Height;
			break;
		default:
			return;
		}
		AdjustPosInfo Info;
		Info.hwnd=m_hwnd;
		Info.rc=rc;
		do {
			Info.fMoved=false;
			::EnumWindows(AdjustPosProc,(LPARAM)&Info);
		} while (Info.fMoved);
		SetPosition(&Info.rc);
	}
}


void CMainWindow::SetAlwaysOnTop(bool fTop)
{
	if (m_fAlwaysOnTop!=fTop) {
		m_fAlwaysOnTop=fTop;
		if (m_hwnd!=NULL) {
			::SetWindowPos(m_hwnd,fTop?HWND_TOPMOST:HWND_NOTOPMOST,
											0,0,0,0,SWP_NOMOVE | SWP_NOSIZE);
			MainMenu.CheckItem(CM_ALWAYSONTOP,fTop);
		}
	}
}


bool CMainWindow::OnCreate()
{
	StatusView.Create(m_hwnd,
		WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS,WS_EX_STATICEDGE,IDC_STATUS);
	StatusView.AddItem(new CTunerStatusItem);
	//StatusView.SetEventHandler(&StatusViewEventHandler);
	StatusView.AddItem(new CChannelStatusItem);
	StatusView.AddItem(new CRecordStatusItem);
	StatusView.AddItem(new CErrorStatusItem);
	StatusView.AddItem(new CSignalLevelStatusItem);
	StatusView.AddItem(new CBufferingStatusItem);
	StatusView.AddItem(new CProgramInfoStatusItem);
	StatusView.AddItem(new CNetworkStatusItem);
	StatusOptions.ApplyOptions();
	StatusView.SetSingleText(TEXT("N..."));

	MainMenu.Create(hInst);
	MainMenu.CheckItem(CM_ALWAYSONTOP,m_fAlwaysOnTop);

	/*
	HMENU hSysMenu;
	hSysMenu=GetSystemMenu(m_hwnd,FALSE);
	AppendMenu(hSysMenu,MFT_SEPARATOR,0,NULL);
	AppendMenu(hSysMenu,MFT_STRING | MFS_ENABLED,SC_ABOUT,
												TEXT("o[W(&A)..."));
	*/

	ToolTip.Initialize(m_hwnd);

	::SetTimer(m_hwnd,TIMER_ID_UPDATE,1000,NULL);
	return true;
}


void CMainWindow::OnCommand(HWND hwnd,int id,HWND hwndCtl,UINT codeNotify)
{
	switch (id) {
	case CM_ALWAYSONTOP:
		SetAlwaysOnTop(!m_fAlwaysOnTop);
		return;

	case CM_RESET:
		CoreEngine.m_DtvEngine.ResetEngine();
		return;

	case CM_RECORD:
	case CM_RECORD_START:
	case CM_RECORD_STOP:
		if (id==CM_RECORD) {
			if (RecordManager.IsRecording() && TvRockDTVTarget.IsOpen()) {
				if (ShowMessage(TEXT("^~܂?"),TEXT("^~̊mF"),
								MB_OKCANCEL | MB_ICONQUESTION | MB_DEFBUTTON2)!=IDOK)
					return;
			}
			if (RecordManager.IsPaused()) {
				SendCommand(CM_RECORD_PAUSE);
				return;
			}
		} else if (id==CM_RECORD_START) {
			if (RecordManager.IsRecording()) {
				if (RecordManager.IsPaused())
					SendCommand(CM_RECORD_PAUSE);
				return;
			}
		} else if (id==CM_RECORD_STOP) {
			if (!RecordManager.IsRecording())
				return;
		}
		if (RecordManager.IsRecording()) {
			TCHAR szText[512],szFileName[MAX_PATH],szSize[32];

			::lstrcpy(szFileName,RecordManager.GetRecordTask()->GetFileName());
			RecordManager.StopRecord();
			ResidentManager.SetStatus(0,CResidentManager::STATUS_RECORDING);
			UInt64ToString(CoreEngine.m_DtvEngine.m_FileWriter.GetWriteSize(),
						   szSize,lengthof(szSize));
			::wsprintf(szText,TEXT("^~ %s (oTCY %s Bytes"),
					   szFileName,szSize);
			if (RecordManager.GetCalcCrc()) {
				::wsprintf(szText+::lstrlen(szText),TEXT(" / CRC32 %08X"),
						   RecordManager.GetCrc());
			}
			::lstrcat(szText,TEXT(")"));
			Logger.AddLog(szText);
			StatusView.UpdateItem(STATUS_ITEM_RECORD);
			MainMenu.EnableItem(CM_RECORDOPTION,true);
			MainMenu.EnableItem(CM_RECORDSTOPTIME,false);
			PluginList.SendRecordStatusChangeEvent();
			CoreEngine.m_DtvEngine.SetDescrambleCurServiceOnly(fDescrambleCurServiceOnly);
			if (m_fStandby)
				CloseTuner();
			if (m_fExitOnRecordingStop)
				PostCommand(CM_EXIT);
		} else {
			if (RecordManager.IsReserved()) {
				if (ShowMessage(
						TEXT("ɐݒ肳Ă^悪܂B\n")
						TEXT("^JnƊ̐ݒ肪j܂B\n")
						TEXT("^JnĂł?"),
						TEXT("^Jn̊mF"),
						MB_OKCANCEL | MB_ICONQUESTION)!=IDOK)
					return;
			}
			AppMain.StartRecord();
		}
		return;

	case CM_RECORD_PAUSE:
		if (RecordManager.IsRecording()) {
			RecordManager.PauseRecord();
			StatusView.UpdateItem(STATUS_ITEM_RECORD);
			Logger.AddLog(RecordManager.IsPaused()?TEXT("^ꎞ~"):TEXT("^ĊJ"));
			PluginList.SendRecordStatusChangeEvent();
		}
		return;

	case CM_RECORDOPTION:
		if (IsWindowEnabled(GetVideoHostWindow())) {
			if (RecordManager.GetFileName()==NULL) {
				TCHAR szFileName[MAX_PATH];

				if (RecordOptions.GetFilePath(szFileName,MAX_PATH))
					RecordManager.SetFileName(szFileName);
			}
			RecordOptions.ApplyOptions(&RecordManager);
			if (RecordManager.RecordDialog(GetVideoHostWindow())) {
				if (RecordManager.IsReserved()) {
					StatusView.UpdateItem(STATUS_ITEM_RECORD);
				} else {
					AppMain.StartReservedRecord();
				}
			}
		}
		return;

	case CM_RECORDSTOPTIME:
		if (RecordManager.ChangeStopTimeDialog(GetVideoHostWindow())) {
			StatusView.UpdateItem(STATUS_ITEM_RECORD);
		}
		return;

	case CM_EXITONRECORDINGSTOP:
		m_fExitOnRecordingStop=!m_fExitOnRecordingStop;
		MainMenu.CheckItem(CM_EXITONRECORDINGSTOP,m_fExitOnRecordingStop);
		return;

	case CM_OPTIONS_RECORD:
		if (IsWindowEnabled(hwnd))
			OptionDialog.ShowDialog(hwnd,COptionDialog::PAGE_RECORD);
		return;

	case CM_OPTIONS:
		if (IsWindowEnabled(hwnd))
			OptionDialog.ShowDialog(hwnd);
		return;

	case CM_STREAMINFO:
		StreamInfo.Show(hwnd);
		return;

	case CM_CLOSE:
		if (m_fStandby) {
			SetStandby(false);
		} else if (ResidentManager.GetResident()) {
			SetStandby(true);
		} else {
			if (CmdLineParser.m_fTvRock) {
				if (ShowMessage(TEXT("TvRock DTV ^[QbgƂē쒆łB\nIĂł?"),TEXT("I̊mF"),MB_OKCANCEL | MB_ICONQUESTION)!=IDOK)
					return;
			}
			PostMessage(WM_CLOSE,0,0);
		}
		return;

	case CM_EXIT:
		PostMessage(WM_CLOSE,0,0);
		return;

	case CM_MINIMIZE:
		SendMessage(WM_SYSCOMMAND,SC_MINIMIZE,0);
		return;

	case CM_SHOW:
		if (m_fStandby) {
			SetStandby(false);
		} else {
			SetWindowVisible();
		}
		return;

	case CM_CHANNEL_UP:
	case CM_CHANNEL_DOWN:
		{
			const CChannelInfo *pInfo=ChannelManager.GetNextChannelInfo(id==CM_CHANNEL_UP);

			if (pInfo!=NULL)
				SendCommand(CM_CHANNELNO_FIRST+pInfo->GetChannelNo()-1);
			else
				SendCommand(CM_CHANNEL_FIRST);
		}
		return;

	case CM_MENU:
		{
			POINT pt;

			pt.x=0;
			pt.y=0;
			::ClientToScreen(hwnd,&pt);
			PopupMenu(&pt);
		}
		return;

	case CM_UDPVIEW:
		UdpOptions.ExecuteNetworkViewer();
		if (!UdpOptions.IsNetworkSending())
			SendCommand(CM_UDPSEND);
		return;

	case CM_UDPSEND:
		if (UdpOptions.IsNetworkSending()) {
			UdpOptions.StopNetworkSend();
		} else {
			int Protocol;
			unsigned int Port=0;

			if (CmdLineParser.m_fUdp) {
				Protocol=CUdpOptions::PROTOCOL_UDP;
				Port=CmdLineParser.m_UdpPort;
			} else if (CmdLineParser.m_fTcp) {
				Protocol=CUdpOptions::PROTOCOL_TCP;
				Port=CmdLineParser.m_TcpPort;
			} else {
				Protocol=CUdpOptions::PROTOCOL_DEFAULT;
				if (UdpOptions.GetProtocol()==CUdpOptions::PROTOCOL_UDP)
					Port=CmdLineParser.m_UdpPort;
				else
					Port=CmdLineParser.m_TcpPort;
			}
			UdpOptions.StartNetworkSend(Protocol,Port);
		}
		MainMenu.CheckItem(CM_UDPSEND,UdpOptions.IsNetworkSending());
		StatusView.UpdateItem(STATUS_ITEM_NETWORK);
		return;

	case CM_NETWORKVIEW:
		UdpOptions.ExecuteNetworkViewer();
		return;

	case CM_CHASINGVIEW:
		UdpOptions.ExecuteChasingViewer();
		return;

	case CM_ABOUT:
		{
			CAboutDialog AboutDialog;

			AboutDialog.Show(hwnd);
		}
		return;

	default:
		if (id>=CM_CHANNELNO_FIRST && id<=CM_CHANNELNO_LAST) {
			int No=id-CM_CHANNELNO_FIRST;

			int Index=ChannelManager.GetCurrentChannelList()->FindChannelNo(No+1);

			if (Index>=0)
				SendCommand(CM_CHANNEL_FIRST+Index);
			return;
		}
		if (id>=CM_CHANNEL_FIRST && id<=CM_CHANNEL_LAST) {
			int Channel=id-CM_CHANNEL_FIRST;

			if (RecordManager.IsRecording()) {
				if (!RecordOptions.ConfirmChannelChange(GetVideoHostWindow()))
					return;
			}/* else if (CmdLineParser.m_fTvRock) {
				if (!ConfirmChannelChange())
					return;
			}*/
			const CChannelList *pChList=ChannelManager.GetCurrentRealChannelList();
			if (pChList==NULL)
				return;
			const CChannelInfo *pChInfo=pChList->GetChannelInfo(Channel);
			if (pChInfo==NULL)
				return;
			AppMain.SetChannel(ChannelManager.GetCurrentSpace(),Channel);
			return;
		}
		if (id>=CM_SERVICE_FIRST && id<=CM_SERVICE_LAST) {

			if (RecordManager.IsRecording()) {
				if (!RecordOptions.ConfirmServiceChange(GetVideoHostWindow(),
														&RecordManager))
					return;
			}
			AppMain.SetServiceByIndex(id-CM_SERVICE_FIRST);
			return;
		}
		if (id>=CM_SPACE_ALL && id<=CM_SPACE_LAST) {
			int Space=id-CM_SPACE_FIRST;

			if (Space!=ChannelManager.GetCurrentSpace()) {
				if (RecordManager.IsRecording()) {
					if (!RecordOptions.ConfirmSpaceChange(GetVideoHostWindow()))
						return;
				}
				AppMain.SetChannel(Space,0);
			}
			return;
		}
		if (id>=CM_DRIVER_FIRST && id<=CM_DRIVER_LAST) {
			int Driver=id-CM_DRIVER_FIRST;

			if (Driver<DriverManager.NumDrivers()) {
				const CDriverInfo *pDriverInfo=DriverManager.GetDriverInfo(Driver);

				if (::lstrcmpi(pDriverInfo->GetFileName(),CoreEngine.GetDriverFileName())!=0) {
					if (RecordManager.IsRecording()) {
						if (!RecordOptions.ConfirmDriverChange(GetVideoHostWindow()))
							return;
					}
					if (AppMain.SetDriver(pDriverInfo->GetFileName()))
						::lstrcpy(szDriverFileName,pDriverInfo->GetFileName());
				}
			}
			return;
		}
		if (id>=CM_PLUGIN_FIRST && id<=CM_PLUGIN_LAST) {
			CPlugin *pPlugin=PluginList.GetPlugin(id-CM_PLUGIN_FIRST);

			if (pPlugin!=NULL)
				pPlugin->Enable(!pPlugin->IsEnabled());
			return;
		}
		if (id>=CM_SPACE_CHANNEL_FIRST && id<=CM_SPACE_CHANNEL_LAST) {
			if (RecordManager.IsRecording()) {
				if (!RecordOptions.ConfirmChannelChange(GetVideoHostWindow()))
					return;
			}
			AppMain.ProcessTunerSelectMenu(id);
			return;
		}
	}
}


BOOL CMainWindow::OnAppCommand(HWND hwnd,HWND hwndFrom,int nCommand,
													UINT uDevice,DWORD dwKeys)
{
#ifndef APPCOMMAND_MEDIA_PLAY
#define APPCOMMAND_OPEN					30
#define APPCOMMAND_CLOSE				31
#define APPCOMMAND_MEDIA_PLAY			46
#define APPCOMMAND_MEDIA_PAUSE			47
#define APPCOMMAND_MEDIA_RECORD			48
#define APPCOMMAND_MEDIA_FAST_FORWARD	49
#define APPCOMMAND_MEDIA_REWIND			50
#define APPCOMMAND_MEDIA_CHANNEL_UP		51
#define APPCOMMAND_MEDIA_CHANNEL_DOWN	52
#endif

	static const struct {
		WORD AppCommand;
		WORD Command;
	} CommandTable [] = {
		{APPCOMMAND_VOLUME_MUTE,			CM_VOLUME_MUTE},
		{APPCOMMAND_VOLUME_DOWN,			CM_VOLUME_DOWN},
		{APPCOMMAND_VOLUME_UP,				CM_VOLUME_UP},
		{APPCOMMAND_MEDIA_NEXTTRACK,		CM_CHANNEL_UP},
		{APPCOMMAND_MEDIA_PREVIOUSTRACK,	CM_CHANNEL_DOWN},
		//{APPCOMMAND_MEDIA_PLAY_PAUSE,		CM_RECORD_PAUSE},
		{APPCOMMAND_MEDIA_STOP,				CM_RECORD_STOP},
		{APPCOMMAND_CLOSE,					CM_CLOSE},
		{APPCOMMAND_MEDIA_PAUSE,			CM_RECORD_PAUSE},
		{APPCOMMAND_MEDIA_RECORD,			CM_RECORD_START},
		{APPCOMMAND_MEDIA_CHANNEL_UP,		CM_CHANNEL_UP},
		{APPCOMMAND_MEDIA_CHANNEL_DOWN,		CM_CHANNEL_DOWN},
	};
	int i;

	for (i=0;i<lengthof(CommandTable);i++) {
		if (nCommand==(int)CommandTable[i].AppCommand) {
			SendCommand(CommandTable[i].Command);
			break;
		}
	}
	return TRUE;
}


void CMainWindow::OnTimer(HWND hwnd,UINT id)
{
	switch (id) {
	case TIMER_ID_UPDATE:
		{
			static unsigned int TimerCount=0;

			DWORD UpdateStatus=CoreEngine.UpdateAsyncStatus();
			DWORD UpdateStatistics=CoreEngine.UpdateStatistics();

			if ((UpdateStatus&CCoreEngine::STATUS_EVENTID)!=0) {
				StatusView.UpdateItem(STATUS_ITEM_PROGRAMINFO);
			}

			if (RecordManager.IsRecording()) {
				if (RecordManager.QueryStop()) {
					SendCommand(CM_RECORD_STOP);
				} else if (!RecordManager.IsPaused()) {
					StatusView.UpdateItem(STATUS_ITEM_RECORD);
				}
			} else {
				if (RecordManager.QueryStart())
					AppMain.StartReservedRecord();
			}

			if ((UpdateStatistics&(CCoreEngine::STATISTIC_ERRORPACKETCOUNT
								 | CCoreEngine::STATISTIC_CONTINUITYERRORPACKETCOUNT
								 | CCoreEngine::STATISTIC_SCRAMBLEPACKETCOUNT))!=0) {
				StatusView.UpdateItem(STATUS_ITEM_ERROR);
			}

			if ((UpdateStatistics&(CCoreEngine::STATISTIC_SIGNALLEVEL
								 | CCoreEngine::STATISTIC_BITRATE))!=0)
				StatusView.UpdateItem(STATUS_ITEM_SIGNALLEVEL);

			if ((UpdateStatistics&CCoreEngine::STATISTIC_STREAMREMAIN)!=0)
				StatusView.UpdateItem(STATUS_ITEM_BUFFERING);

			TimerCount++;
		}
		break;

	case TIMER_ID_WHEELCHANNELCHANGE:
		{
			const CChannelInfo *pInfo=ChannelManager.GetChangingChannelInfo();

			fWheelChannelChanging=false;
			ChannelManager.SetChangingChannel(-1);
			if (pInfo!=NULL)
				SendCommand(CM_CHANNELNO_FIRST+pInfo->GetChannelNo()-1);
			::KillTimer(hwnd,TIMER_ID_WHEELCHANNELCHANGE);
		}
		break;

	case TIMER_ID_INFOTIP:
		ToolTip.Hide();
		::KillTimer(hwnd,TIMER_ID_INFOTIP);
		break;
	}
}


void CMainWindow::OnMouseWheel(WPARAM wParam,LPARAM lParam)
{
	/*
	int Mode;
	POINT pt;
	RECT rc;

	pt.x=GET_X_LPARAM(lParam);
	pt.y=GET_Y_LPARAM(lParam);
	StatusView.GetScreenPosition(&rc);
	if (PtInRect(&rc,pt)) {
		switch (StatusView.GetCurItem()) {
		case STATUS_ITEM_CHANNEL:		Mode=WHEEL_MODE_CHANNEL;	break;
		}
	}
	switch (Mode) {
	case WHEEL_MODE_CHANNEL:
	*/
		if (!CmdLineParser.m_fTvRock) {
			bool fUp;
			const CChannelInfo *pInfo;

			if (fWheelChannelReverse)
				fUp=GET_WHEEL_DELTA_WPARAM(wParam)>0;
			else
				fUp=GET_WHEEL_DELTA_WPARAM(wParam)<0;
			pInfo=ChannelManager.GetNextChannelInfo(fUp);
			if (pInfo!=NULL) {
				fWheelChannelChanging=true;
				ChannelManager.SetChangingChannel(ChannelManager.FindChannelInfo(pInfo));
				StatusView.UpdateItem(STATUS_ITEM_CHANNEL);
				SetTimer(m_hwnd,TIMER_ID_WHEELCHANNELCHANGE,WheelChannelDelay,NULL);
			}
		}
/*
		break;
	}
*/
}


void CMainWindow::PopupMenu(const POINT *pPos/*=NULL*/)
{
	POINT pt;

	if (pPos!=NULL)
		pt=*pPos;
	else
		::GetCursorPos(&pt);
	MainMenu.Popup(TPM_RIGHTBUTTON,pt.x,pt.y,m_hwnd,true);
}




/*
	EBhE̒[ۂɕ\Ă邩肷
	ꂾƕsS(ɍőOʂ̃EBhElĂȂ)
*/
static bool IsWindowEdgeVisible(HWND hwnd,HWND hwndTop,const RECT *pRect,HWND hwndTarget)
{
	RECT rc,rcEdge;
	HWND hwndNext;

	if (hwndTop==hwnd || hwndTop==NULL)
		return true;
	GetWindowRect(hwndTop,&rc);
	hwndNext=GetNextWindow(hwndTop,GW_HWNDNEXT);
	if (hwndTop==hwndTarget || !IsWindowVisible(hwndTop)
			|| rc.left==rc.right || rc.top==rc.bottom)
		return IsWindowEdgeVisible(hwnd,hwndNext,pRect,hwndTarget);
	if (pRect->top==pRect->bottom) {
		if (rc.top<=pRect->top && rc.bottom>pRect->top) {
			if (rc.left<=pRect->left && rc.right>=pRect->right)
				return false;
			if (rc.left<=pRect->left && rc.right>pRect->left) {
				rcEdge=*pRect;
				rcEdge.right=min(rc.right,pRect->right);
				return IsWindowEdgeVisible(hwnd,hwndNext,&rcEdge,hwndTarget);
			} else if (rc.left>pRect->left && rc.right>=pRect->right) {
				rcEdge=*pRect;
				rcEdge.left=rc.left;
				return IsWindowEdgeVisible(hwnd,hwndNext,&rcEdge,hwndTarget);
			} else if (rc.left>pRect->left && rc.right<pRect->right) {
				rcEdge=*pRect;
				rcEdge.right=rc.left;
				if (IsWindowEdgeVisible(hwnd,hwndNext,&rcEdge,hwndTarget))
					return true;
				rcEdge.left=rc.right;
				rcEdge.right=pRect->right;
				return IsWindowEdgeVisible(hwnd,hwndNext,&rcEdge,hwndTarget);
			}
		}
	} else {
		if (rc.left<=pRect->left && rc.right>pRect->left) {
			if (rc.top<=pRect->top && rc.bottom>=pRect->bottom)
				return false;
			if (rc.top<=pRect->top && rc.bottom>pRect->top) {
				rcEdge=*pRect;
				rcEdge.bottom=min(rc.bottom,pRect->bottom);
				return IsWindowEdgeVisible(hwnd,hwndNext,&rcEdge,hwndTarget);
			} else if (rc.top>pRect->top && rc.bottom>=pRect->bottom) {
				rcEdge=*pRect;
				rcEdge.top=rc.top;
				return IsWindowEdgeVisible(hwnd,hwndNext,&rcEdge,hwndTarget);
			} else if (rc.top>pRect->top && rc.bottom<pRect->bottom) {
				rcEdge=*pRect;
				rcEdge.bottom=rc.top;
				if (IsWindowEdgeVisible(hwnd,hwndNext,&rcEdge,hwndTarget))
					return true;
				rcEdge.top=rc.bottom;
				rcEdge.bottom=pRect->bottom;
				return IsWindowEdgeVisible(hwnd,hwndNext,&rcEdge,hwndTarget);
			}
		}
	}
	return IsWindowEdgeVisible(hwnd,hwndNext,pRect,hwndTarget);
}


struct SnapWindowInfo {
	HWND hwnd;
	RECT rcOriginal;
	RECT rcNearest;
};

static BOOL CALLBACK SnapWindowProc(HWND hwnd,LPARAM lParam)
{
	SnapWindowInfo *pInfo=reinterpret_cast<SnapWindowInfo*>(lParam);

	if (IsWindowVisible(hwnd) && hwnd!=pInfo->hwnd) {
		RECT rc,rcEdge;

		GetWindowRect(hwnd,&rc);
		if (rc.right>rc.left && rc.bottom>rc.top) {
			if (rc.top<pInfo->rcOriginal.bottom && rc.bottom>pInfo->rcOriginal.top) {
				if (abs(rc.left-pInfo->rcOriginal.right)<abs(pInfo->rcNearest.right)) {
					rcEdge.left=rc.left;
					rcEdge.right=rc.left;
					rcEdge.top=max(rc.top,pInfo->rcOriginal.top);
					rcEdge.bottom=min(rc.bottom,pInfo->rcOriginal.bottom);
					if (IsWindowEdgeVisible(hwnd,GetTopWindow(GetDesktopWindow()),&rcEdge,pInfo->hwnd))
						pInfo->rcNearest.right=rc.left-pInfo->rcOriginal.right;
				}
				if (abs(rc.right-pInfo->rcOriginal.left)<abs(pInfo->rcNearest.left)) {
					rcEdge.left=rc.right;
					rcEdge.right=rc.right;
					rcEdge.top=max(rc.top,pInfo->rcOriginal.top);
					rcEdge.bottom=min(rc.bottom,pInfo->rcOriginal.bottom);
					if (IsWindowEdgeVisible(hwnd,GetTopWindow(GetDesktopWindow()),&rcEdge,pInfo->hwnd))
						pInfo->rcNearest.left=rc.right-pInfo->rcOriginal.left;
				}
			}
			if (rc.left<pInfo->rcOriginal.right && rc.right>pInfo->rcOriginal.left) {
				if (abs(rc.top-pInfo->rcOriginal.bottom)<abs(pInfo->rcNearest.bottom)) {
					rcEdge.left=max(rc.left,pInfo->rcOriginal.left);
					rcEdge.right=min(rc.right,pInfo->rcOriginal.right);
					rcEdge.top=rc.top;
					rcEdge.bottom=rc.top;
					if (IsWindowEdgeVisible(hwnd,GetTopWindow(GetDesktopWindow()),&rcEdge,pInfo->hwnd))
						pInfo->rcNearest.bottom=rc.top-pInfo->rcOriginal.bottom;
				}
				if (abs(rc.bottom-pInfo->rcOriginal.top)<abs(pInfo->rcNearest.top)) {
					rcEdge.left=max(rc.left,pInfo->rcOriginal.left);
					rcEdge.right=min(rc.right,pInfo->rcOriginal.right);
					rcEdge.top=rc.bottom;
					rcEdge.bottom=rc.bottom;
					if (IsWindowEdgeVisible(hwnd,GetTopWindow(GetDesktopWindow()),&rcEdge,pInfo->hwnd))
						pInfo->rcNearest.top=rc.bottom-pInfo->rcOriginal.top;
				}
			}
		}
	}
	return TRUE;
}


static void SnapWindow(HWND hwnd,RECT *prc)
{
	HMONITOR hMonitor;
	RECT rc;
	SnapWindowInfo Info;
	int XOffset,YOffset;

	hMonitor=MonitorFromWindow(hwnd,MONITOR_DEFAULTTONEAREST);
	if (hMonitor!=NULL) {
		MONITORINFO mi;

		mi.cbSize=sizeof(MONITORINFO);
		GetMonitorInfo(hMonitor,&mi);
		rc=mi.rcMonitor;
	} else {
		rc.left=0;
		rc.top=0;
		rc.right=GetSystemMetrics(SM_CXSCREEN);
		rc.bottom=GetSystemMetrics(SM_CYSCREEN);
	}
	Info.hwnd=hwnd;
	Info.rcOriginal=*prc;
	Info.rcNearest.left=rc.left-prc->left;
	Info.rcNearest.top=rc.top-prc->top;
	Info.rcNearest.right=rc.right-prc->right;
	Info.rcNearest.bottom=rc.bottom-prc->bottom;
	EnumWindows(SnapWindowProc,reinterpret_cast<LPARAM>(&Info));
	if (abs(Info.rcNearest.left)<abs(Info.rcNearest.right)
			|| Info.rcNearest.left==Info.rcNearest.right)
		XOffset=Info.rcNearest.left;
	else if (abs(Info.rcNearest.left)>abs(Info.rcNearest.right))
		XOffset=Info.rcNearest.right;
	else
		XOffset=0;
	if (abs(Info.rcNearest.top)<abs(Info.rcNearest.bottom)
			|| Info.rcNearest.top==Info.rcNearest.bottom)
		YOffset=Info.rcNearest.top;
	else if (abs(Info.rcNearest.top)>abs(Info.rcNearest.bottom))
		YOffset=Info.rcNearest.bottom;
	else
		YOffset=0;
	if (abs(XOffset)<=SnapAtWindowEdgeMargin)
		prc->left+=XOffset;
	if (abs(YOffset)<=SnapAtWindowEdgeMargin)
		prc->top+=YOffset;
	prc->right=prc->left+(Info.rcOriginal.right-Info.rcOriginal.left);
	prc->bottom=prc->top+(Info.rcOriginal.bottom-Info.rcOriginal.top);
}


CMainWindow *CMainWindow::GetThis(HWND hwnd)
{
	return static_cast<CMainWindow*>(GetBasicWindow(hwnd));
}


LRESULT CALLBACK CMainWindow::WndProc(HWND hwnd,UINT uMsg,
												WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_CREATE:
		{
			CMainWindow *pThis=static_cast<CMainWindow*>(CBasicWindow::OnCreate(hwnd,lParam));

			if (!pThis->OnCreate())
				return -1;
		}
		return 0;

	case WM_SIZE:
		{
			CMainWindow *pThis=GetThis(hwnd);

			if (wParam==SIZE_MINIMIZED) {
				ResidentManager.SetStatus(CResidentManager::STATUS_MINIMIZED,
										  CResidentManager::STATUS_MINIMIZED);
			} else if ((ResidentManager.GetStatus()&CResidentManager::STATUS_MINIMIZED)!=0) {
				pThis->SetWindowVisible();
			}

			int Width=LOWORD(lParam);//,Height=HIWORD(lParam);

			Width-=CAPTION_WIDTH;
			StatusView.SetPosition(CAPTION_WIDTH,0,Width,StatusView.GetHeight());
		}
		return 0;

	case WM_GETMINMAXINFO:
		{
			CMainWindow *pThis=GetThis(hwnd);
			if (pThis==NULL)
				break;

			LPMINMAXINFO pmmi=reinterpret_cast<LPMINMAXINFO>(lParam);
			RECT rc;

			::SetRect(&rc,0,0,CAPTION_WIDTH+16,StatusView.GetHeight());
			::AdjustWindowRectEx(&rc,GetWindowStyle(hwnd),FALSE,GetWindowExStyle(hwnd));
			pmmi->ptMinTrackSize.x=rc.right-rc.left;
			pmmi->ptMinTrackSize.y=rc.bottom-rc.top;
			pmmi->ptMaxTrackSize.y=rc.bottom-rc.top;
		}
		return 0;

	case WM_PAINT:
		{
			CMainWindow *pThis=GetThis(hwnd);
			PAINTSTRUCT ps;

			::BeginPaint(hwnd,&ps);
			if (TvRockDTVTarget.IsOpen()) {
				static const COLORREF CaptionColor[8] = {
					RGB(0x00, 0xFF ,0x7F),
					RGB(0x00, 0x7F, 0xFF),
					RGB(0x00, 0x00, 0xFF),
					RGB(0x7F, 0x00, 0xFF),
					RGB(0xFF, 0x00, 0x7F),
					RGB(0xFF, 0x00, 0x00),
					RGB(0xFF, 0x7F, 0x00),
					RGB(0x7F, 0xFF, 0x00),
				};
				RECT rc;
				HFONT hfontOld;
				int OldBkMode;
				COLORREF crOldTextColor;
				TCHAR Char;

				::GetClientRect(hwnd,&rc);
				rc.right=CAPTION_WIDTH;
				DrawUtil::FillGradient(ps.hdc,&rc,
					CaptionColor[TvRockDTVTarget.GetDeviceID()%8],RGB(0,0,0));
				hfontOld=static_cast<HFONT>(::SelectObject(ps.hdc,pThis->m_hfontCaption));
				OldBkMode=::SetBkMode(ps.hdc,TRANSPARENT);
				crOldTextColor=::SetTextColor(ps.hdc,RGB(255,255,255));
				Char=(TCHAR)(TvRockDTVTarget.GetDeviceID()+'A');
				::DrawText(ps.hdc,&Char,1,&rc,DT_CENTER | DT_VCENTER | DT_SINGLELINE);
				::SelectObject(ps.hdc,hfontOld);
				::SetBkMode(ps.hdc,OldBkMode);
				::SetTextColor(ps.hdc,crOldTextColor);
			} else {
				::FillRect(ps.hdc,&ps.rcPaint,reinterpret_cast<HBRUSH>(COLOR_3DFACE+1));
			}
			::EndPaint(hwnd,&ps);
		}
		return 0;

	case WM_RBUTTONDOWN:
		{
			CMainWindow *pThis=GetThis(hwnd);
			/*
			POINT pt;

			pt.x=GET_X_LPARAM(lParam);
			pt.y=GET_Y_LPARAM(lParam);
			::ClientToScreen(hwnd,&pt);
			pThis->PopupMenu(&pt);
			*/
			pThis->PopupMenu();
		}
		return 0;

	case WM_NCLBUTTONDOWN:
		if (wParam!=HTCAPTION)
			break;
		ForegroundWindow(hwnd);
	case WM_LBUTTONDOWN:
		{
			CMainWindow *pThis=GetThis(hwnd);

			/*
			pThis->m_ptDragStartPos.x=GET_X_LPARAM(lParam);
			pThis->m_ptDragStartPos.y=GET_Y_LPARAM(lParam);
			ClientToScreen(hwnd,&pThis->m_ptDragStartPos);
			*/
			::GetCursorPos(&pThis->m_ptDragStartPos);
			::GetWindowRect(hwnd,&pThis->m_rcDragStart);
			::SetCapture(hwnd);
		}
		return 0;

	case WM_NCLBUTTONUP:
	case WM_LBUTTONUP:
		if (::GetCapture()==hwnd) {
			::ReleaseCapture();
		}
		return 0;

	case WM_MOUSEMOVE:
		{
			CMainWindow *pThis=GetThis(hwnd);

			if (GetCapture()==hwnd) {
				POINT pt;
				RECT rcOld,rc;

				/*
				pt.x=GET_X_LPARAM(lParam);
				pt.y=GET_Y_LPARAM(lParam);
				ClientToScreen(hwnd,&pt);
				*/
				::GetWindowRect(hwnd,&rcOld);
				::GetCursorPos(&pt);
				rc.left=pThis->m_rcDragStart.left+(pt.x-pThis->m_ptDragStartPos.x);
				rc.top=pThis->m_rcDragStart.top+(pt.y-pThis->m_ptDragStartPos.y);
				rc.right=rc.left+(pThis->m_rcDragStart.right-pThis->m_rcDragStart.left);
				rc.bottom=rc.top+(pThis->m_rcDragStart.bottom-pThis->m_rcDragStart.top);
				if (::GetKeyState(VK_SHIFT)<0?!fSnapAtWindowEdge:fSnapAtWindowEdge)
					SnapWindow(hwnd,&rc);
				pThis->SetPosition(&rc);
			}
		}
		return 0;

	case WM_LBUTTONDBLCLK:
		::ShowWindow(hwnd,SW_MINIMIZE);
		return 0;

	case WM_SETCURSOR:
		{
			int Code=LOWORD(lParam);
			LPCTSTR pszCursor=NULL;

			switch (Code) {
			case HTTOP:
			case HTBOTTOM:
				pszCursor=IDC_ARROW;
				break;
			case HTLEFT:
			case HTTOPLEFT:
			case HTBOTTOMLEFT:
			case HTRIGHT:
			case HTTOPRIGHT:
			case HTBOTTOMRIGHT:
				pszCursor=IDC_SIZEWE;
				break;
			}
			if (pszCursor) {
				::SetCursor(::LoadCursor(NULL,pszCursor));
				return TRUE;
			}
		}
		break;

	case WM_SYSKEYDOWN:
		if (wParam!=VK_F10)
			break;
	case WM_KEYDOWN:
		{
			CMainWindow *pThis=GetThis(hwnd);
			int Command;

			if (wParam>=VK_F1 && wParam<=VK_F12) {
				if (!fFunctionKeyChangeChannel)
					break;
				Command=CM_CHANNELNO_FIRST+((int)wParam-VK_F1);
			} else if (wParam>=VK_NUMPAD0 && wParam<=VK_NUMPAD9) {
				if (!fNumPadChangeChannel)
					break;
				if (wParam==VK_NUMPAD0)
					Command=CM_CHANNELNO_FIRST+9;
				else
					Command=CM_CHANNELNO_FIRST+((int)wParam-VK_NUMPAD1);
			} else if (wParam>='0' && wParam<='9') {
				if (!fDigitKeyChangeChannel)
					break;
				if (wParam=='0')
					Command=CM_CHANNELNO_FIRST+9;
				else
					Command=CM_CHANNELNO_FIRST+((int)wParam-'1');
			} else {
				static const struct {
					WORD KeyCode;
					WORD Command;
				} CommandMap[] = {
					{'R',		CM_RECORD},
					{'T',		CM_ALWAYSONTOP},
					{'V',		CM_UDPVIEW},
					{VK_LEFT,	CM_CHANNEL_DOWN},
					{VK_RIGHT,	CM_CHANNEL_UP},
				};
				int i;

				for (i=0;i<lengthof(CommandMap);i++) {
					if (CommandMap[i].KeyCode==wParam)
						break;
				}
				if (i==lengthof(CommandMap))
					break;
				Command=CommandMap[i].Command;
			}
			pThis->SendCommand(Command);
		}
		return 0;

	case WM_MOUSEWHEEL:
		GetThis(hwnd)->OnMouseWheel(wParam,lParam);
		return 0;

	case WM_INITMENUPOPUP:
		if ((HMENU)wParam==MainMenu.GetSubMenu(CMainMenu::SUBMENU_PLUGIN)) {
			PluginList.SetMenu((HMENU)wParam);
			return 0;
		}
		break;

	case WM_SYSCOMMAND:
		switch ((wParam&0xFFFFFFF0UL)) {
		/*
		case SC_ABOUT:
			{
				CAboutDialog AboutDialog;

				AboutDialog.Show(hwnd);
			}
			return 0;
		*/

		case SC_CLOSE:
			GetThis(hwnd)->SendCommand(CM_CLOSE);
			return 0;
		}
		break;

	case WM_APPCOMMAND:
		return GetThis(hwnd)->OnAppCommand(hwnd,(HWND)wParam,
					GET_APPCOMMAND_LPARAM(lParam),GET_DEVICE_LPARAM(lParam),
					GET_KEYSTATE_LPARAM(lParam));

	case WM_POWERBROADCAST:
		if (wParam==PBT_APMSUSPEND) {
			CMainWindow *pThis=GetThis(hwnd);

			Logger.AddLog(TEXT("TXyhւ̈ڍsbZ[WM܂B"));
			pThis->m_RestoreChannelSpec.Store(&ChannelManager);
			pThis->CloseTuner();
		} else if (wParam==PBT_APMRESUMESUSPEND) {
			CMainWindow *pThis=GetThis(hwnd);

			Logger.AddLog(TEXT("TXyh̕AbZ[WM܂B"));
			pThis->OpenTuner();
		}
		break;

	case WM_APP_SERVICEUPDATE:
		{
			CServiceUpdateInfo *pInfo=reinterpret_cast<CServiceUpdateInfo*>(lParam);
			HMENU hmenu;
			int i;

			hmenu=MainMenu.GetSubMenu(CMainMenu::SUBMENU_SERVICE);
			ClearMenu(hmenu);
			for (i=0;i<pInfo->m_NumServices;i++) {
				TCHAR szServiceName[256];
				if (pInfo->m_pServiceList[i].szServiceName[0]!='\0') {
					::wsprintf(szServiceName,TEXT("%d: %s"),i+1,
							   pInfo->m_pServiceList[i].szServiceName);
				} else {
					::wsprintf(szServiceName,TEXT("T[rX%d"),i+1);
				}
				if (pInfo->m_pServiceList[i].VideoStreamType==0x1B)
					::lstrcat(szServiceName,TEXT(" (ZO)"));
				::AppendMenu(hmenu,MFT_STRING | MFS_ENABLED,CM_SERVICE_FIRST+i,szServiceName);
			}
			if (pInfo->m_CurService>=0)
				MainMenu.CheckRadioItem(CM_SERVICE_FIRST,
										CM_SERVICE_FIRST+pInfo->m_NumServices-1,
										CM_SERVICE_FIRST+pInfo->m_CurService);

			if (pInfo->m_NumServices>0 && pInfo->m_CurService>=0) {
				const CChannelInfo *pChInfo=ChannelManager.GetCurrentRealChannelInfo();
				WORD ServiceID,TransportStreamID;

				TransportStreamID=pInfo->m_TransportStreamID;
				ServiceID=pInfo->m_pServiceList[pInfo->m_CurService].ServiceID;
				/*
				Logger.AddLog(TEXT("T[rX̍\ω܂B(TSID %d)"),
							  TransportStreamID);
				TCHAR szText[128],*p;
				p=szText+::wsprintf(szText,TEXT("(SID "));
				for (int i=0;i<pInfo->m_NumServices;i++)
					p+=::wsprintf(p,TEXT("%d,"),pInfo->m_pServiceList[i].ServiceID);
				*(p-1)=')';
				Logger.AddLog(szText);
				*/
				if (TransportStreamID!=0 && ServiceID!=0
						&& !CoreEngine.IsUDPDriver()
						&& (pChInfo==NULL
						|| (pChInfo->GetTransportStreamID()!=0
						&& pChInfo->GetTransportStreamID()!=TransportStreamID))) {
					// O`lύXꂽA
					// hCoJꂽƂ̃ftHg`l
					AppMain.FollowChannelChange(TransportStreamID,ServiceID);
				}/* else if (pChInfo!=NULL && ServiceID!=0) {
					// T[rXI
					// `l؂ւ͂܂PMTĂȂ̂
					// T[rX̑IłȂ
					WORD SID=0;

					if (ChannelManager.GetCurrentServiceID()!=0) {
						SID=ChannelManager.GetCurrentServiceID();
					} else if (ChannelManager.GetCurrentService()<0
							&& pChInfo->GetServiceID()!=0) {
						SID=pChInfo->GetServiceID();
					}
					if (SID!=0 && SID!=ServiceID) {
						if (AppMain.SetServiceByID(SID,&pInfo->m_CurService))
							ChannelManager.SetCurrentService(pInfo->m_CurService);
					}
				}*/
				/*
				if (pChInfo!=NULL && !CoreEngine.IsUDPDriver()) {
					// `l̏XV
					// Â`lݒt@CɂNIDTSID̏񂪊܂܂ĂȂ
					WORD NetworkID=pInfo->m_NetworkID;

					for (i=0;i<pInfo->m_NumServices;i++) {
						ServiceID=pInfo->m_pServiceList[i].ServiceID;
						if (ServiceID!=0) {
							ChannelManager.UpdateStreamInfo(
								pChInfo->GetSpace(),
								pChInfo->GetChannelIndex(),i,
								NetworkID,TransportStreamID,ServiceID);
						}
					}
				}
				*/
				TvRockDTVTarget.OnEvent(CTvRockDTVTarget::EVENT_SERVICEUPDATE);
				PluginList.SendServiceUpdateEvent();
			}
			delete pInfo;
		}
		return 0;

	case WM_APP_TRAYICON:
		switch (lParam) {
		case WM_RBUTTONDOWN:
			{
				CMainWindow *pThis=GetThis(hwnd);
				POINT pt;
				HMENU hmenu;

				hmenu=::LoadMenu(hInst,MAKEINTRESOURCE(IDM_TRAY));
				::EnableMenuItem(hmenu,CM_SHOW,
					MF_BYCOMMAND | (pThis->m_fStandby || pThis->IsMinimizeToTray()?MFS_ENABLED:MFS_GRAYED));
				::GetCursorPos(&pt);
				// 񑩂KvȗR͈ȉQ
				// http://support.microsoft.com/kb/135788/en-us
				ForegroundWindow(hwnd);				// 
				::TrackPopupMenu(GetSubMenu(hmenu,0),TPM_RIGHTBUTTON,pt.x,pt.y,0,hwnd,NULL);
				::PostMessage(hwnd,WM_NULL,0,0);	// 
				::DestroyMenu(hmenu);
			}
			break;

		case WM_LBUTTONDOWN:
			{
				CMainWindow *pThis=GetThis(hwnd);

				pThis->SendCommand(CM_SHOW);
			}
			break;
		}
		return 0;

	case WM_APP_FILEWRITEERROR:
		Logger.AddLog(TEXT("t@C֏o܂B"));
		GetThis(hwnd)->PopupErrorMessage(TEXT("t@Cւ̏oŃG[܂B"));
		return 0;

	case WM_CLOSE:
		{
			CMainWindow *pThis=GetThis(hwnd);

			if (!pThis->ConfirmExit())
				return 0;

			pThis->m_fClosing=true;
		}
		break;

	case WM_DESTROY:
		{
			CMainWindow *pThis=GetThis(hwnd);
			HANDLE hEvent,hThread;

			hEvent=::CreateEvent(NULL,FALSE,FALSE,NULL);
			hThread=::CreateThread(NULL,0,ExitWatchThread,hEvent,0,NULL);

			::KillTimer(hwnd,TIMER_ID_UPDATE);

			ResidentManager.Finalize();
			MainMenu.Destroy();
			ToolTip.Finalize();

			RestoreChannelInfo.Space=-1;
			RestoreChannelInfo.Channel=-1;
			RestoreChannelInfo.ServiceID=0;
			::lstrcpy(RestoreChannelInfo.szDriverName,CoreEngine.GetDriverFileName());
			if (!CoreEngine.IsUDPDriver()) {
				const CChannelInfo *pInfo=ChannelManager.GetCurrentChannelInfo();

				if (pInfo!=NULL) {
					RestoreChannelInfo.Space=pInfo->GetSpace();
					RestoreChannelInfo.Channel=pInfo->GetChannelIndex();
					RestoreChannelInfo.ServiceID=pInfo->GetServiceID();
				}
			}

			CoreEngine.m_DtvEngine.SetTracer(&Logger);
			CoreEngine.Close();
			CoreEngine.m_DtvEngine.SetTracer(NULL);
			CoreEngine.m_DtvEngine.m_BonSrcDecoder.SetTracer(NULL);

			if (TvRockDTVTarget.IsOpen()) {
				TvRockDTVTarget.Finalize();
				TvRockDTVTarget.SetTracer(NULL);
			}
			PluginOptions.StorePluginOptions();
			Logger.AddLog(TEXT("vOCJĂ܂..."));
			PluginList.FreePlugins();
			AppMain.Finalize();

			::SetEvent(hEvent);
			if (::WaitForSingleObject(hThread,10000)!=WAIT_OBJECT_0)
				::TerminateThread(hThread,-1);
			::CloseHandle(hThread);
			::CloseHandle(hEvent);

			pThis->OnDestroy();
			::PostQuitMessage(0);
		}
		return 0;

	HANDLE_MSG(hwnd,WM_COMMAND,GetThis(hwnd)->OnCommand);
	HANDLE_MSG(hwnd,WM_TIMER,GetThis(hwnd)->OnTimer);

	default:
		if (ResidentManager.HandleMessage(uMsg,wParam,lParam))
			return 0;
	}
	return ::DefWindowProc(hwnd,uMsg,wParam,lParam);
}


DWORD WINAPI CMainWindow::ExitWatchThread(LPVOID lpParameter)
{
	HANDLE hEvent=lpParameter;

	if (::WaitForSingleObject(hEvent,30000)!=WAIT_OBJECT_0) {
		Logger.AddLog(TEXT("I^CAEg܂BvZXIɏI܂B"));
		::ExitProcess(-1);
	}
	return 0;
}


void CMainWindow::SetWindowVisible()
{
	bool fRestore=false;

	if ((ResidentManager.GetStatus()&CResidentManager::STATUS_MINIMIZED)!=0) {
		ResidentManager.SetStatus(0,CResidentManager::STATUS_MINIMIZED);
		fRestore=true;
	}
	if (!GetVisible()) {
		SetVisible(true);
		ForegroundWindow(m_hwnd);
		Update();
		fRestore=true;
	} else if (::IsIconic(m_hwnd)) {
		Show(SW_RESTORE);
		Update();
		fRestore=true;
	} else {
		ForegroundWindow(m_hwnd);
	}
	if (fRestore && (m_fStandbyInit || m_fMinimizeInit)) {
		AdjustPos();
		if (m_fMinimizeInit)
			m_fMinimizeInit=false;
	}
}


bool CMainWindow::SetStandby(bool fStandby)
{
	if (m_fStandby!=fStandby) {
		if (fStandby) {
			SetVisible(false);
			m_RestoreChannelSpec.Store(&ChannelManager);
			if (!RecordManager.IsRecording())
				CloseTuner();
			PluginList.SendStandbyEvent(true);
		} else {
			SetWindowVisible();
			::SetCursor(LoadCursor(NULL,IDC_WAIT));
			PluginList.SendStandbyEvent(false);
			if (m_fStandbyInit) {
				TCHAR szText[1024];

				if (CoreEngine.GetDriverFileName()[0]!='\0') {
					StatusView.SetSingleText(TEXT("hCo̓ǂݍݒ..."));
					if (CoreEngine.LoadDriver()) {
						Logger.AddLog(TEXT("%s ǂݍ݂܂B"),CoreEngine.GetDriverFileName());
						if (!CoreEngine.OpenDriver()) {
							Logger.AddLog(TEXT("BonDriveȑł܂B"));
							PopupErrorMessage(TEXT("BonDriveȑł܂B"));
						}
					} else {
						::wsprintf(szText,TEXT("\"%s\" ǂݍ߂܂B"),CoreEngine.GetDriverFileName());
						Logger.AddLog(szText);
						PopupErrorMessage(szText);
					}
				}
				CoreEngine.m_DtvEngine.SetTracer(&StatusView);
				if (!CoreEngine.OpenBcasCard()) {
					CoreEngine.FormatLastErrorText(szText,lengthof(szText));
					Logger.AddLog(TEXT("J[h[_I[vł܂B"));
					PopupErrorMessage(szText);
				}
				CoreEngine.m_DtvEngine.SetTracer(NULL);
				StatusView.SetSingleText(NULL);
				AppMain.InitializeChannel();
				m_fStandbyInit=false;
			}
			ForegroundWindow(m_hwnd);
			OpenTuner();
			::SetCursor(LoadCursor(NULL,IDC_ARROW));
		}
		m_fStandby=fStandby;
	}
	return true;
}


bool CMainWindow::InitStandby()
{
	if (CoreEngine.GetDriverFileName()[0]!='\0')
		m_fSrcFilterReleased=true;
	if (RestoreChannelInfo.Space>=0 && RestoreChannelInfo.Channel>=0) {
		const CChannelList *pList=ChannelManager.GetChannelList(RestoreChannelInfo.Space);
		if (pList!=NULL) {
			int Index=pList->Find(RestoreChannelInfo.Space,
								  RestoreChannelInfo.Channel,
								  RestoreChannelInfo.ServiceID);
			if (Index>=0) {
				m_RestoreChannelSpec.SetSpace(RestoreChannelInfo.Space);
				m_RestoreChannelSpec.SetChannel(Index);
				m_RestoreChannelSpec.SetServiceID(RestoreChannelInfo.ServiceID);
			}
		}
	}
	ResidentManager.SetResident(true);
	m_fStandby=true;
	m_fStandbyInit=true;
	return true;
}


bool CMainWindow::InitMinimize()
{
	if (RestoreChannelInfo.Space>=0 && RestoreChannelInfo.Channel>=0) {
		const CChannelList *pList=ChannelManager.GetChannelList(RestoreChannelInfo.Space);
		if (pList!=NULL) {
			int Index=pList->Find(RestoreChannelInfo.Space,
								  RestoreChannelInfo.Channel,
								  RestoreChannelInfo.ServiceID);
			if (Index>=0) {
				m_RestoreChannelSpec.SetSpace(RestoreChannelInfo.Space);
				m_RestoreChannelSpec.SetChannel(Index);
				m_RestoreChannelSpec.SetServiceID(RestoreChannelInfo.ServiceID);
			}
		}
	}
	ResidentManager.SetStatus(CResidentManager::STATUS_MINIMIZED,
							  CResidentManager::STATUS_MINIMIZED);
	if (!ResidentManager.GetMinimizeToTray())
		::ShowWindow(m_hwnd,SW_SHOWMINNOACTIVE/*SW_SHOWMINIMIZED*/);
	m_fMinimizeInit=true;
	return true;
}


bool CMainWindow::IsMinimizeToTray() const
{
	return ResidentManager.GetMinimizeToTray()
		&& (ResidentManager.GetStatus()&CResidentManager::STATUS_MINIMIZED)!=0;
}


void CMainWindow::CloseTuner()
{
	if (CoreEngine.IsDriverOpen()) {
		CoreEngine.UnloadDriver();
		ChannelManager.SetCurrentChannel(ChannelManager.GetCurrentSpace(),-1);
		m_fSrcFilterReleased=true;
		Logger.AddLog(TEXT("hCo܂B"));
	}
}


bool CMainWindow::OpenTuner()
{
	bool fOK=true;

	if (!CoreEngine.IsBcasCardOpen()) {
		if (!CoreEngine.OpenBcasCard()) {
			Logger.AddLog(TEXT("J[h[_I[vł܂B"));
		}
	}
	if (m_fSrcFilterReleased) {
		TCHAR szText[1024];

		CoreEngine.m_DtvEngine.SetTracer(&StatusView);
		if (!CoreEngine.IsDriverLoaded()) {
			if (!CoreEngine.LoadDriver()) {
				::wsprintf(szText,TEXT("\"%s\" ǂݍ߂܂B"),CoreEngine.GetDriverFileName());
				Logger.AddLog(szText);
				PopupErrorMessage(szText);
				fOK=false;
			}
		}
		if (fOK && !CoreEngine.OpenDriver()) {
			CoreEngine.FormatLastErrorText(szText,lengthof(szText));
			Logger.AddLog(CoreEngine.GetLastErrorText());
			PopupErrorMessage(szText);
			fOK=false;
		}
		m_fSrcFilterReleased=false;
		CoreEngine.m_DtvEngine.SetTracer(NULL);
		StatusView.SetSingleText(NULL);
		if (fOK) {
			AppMain.SetChannel(m_RestoreChannelSpec.GetSpace(),
							   m_RestoreChannelSpec.GetChannel(),
							   m_RestoreChannelSpec.GetServiceID());
		}
	}
	return fOK;
}


bool CMainWindow::ConfirmExit()
{
	return RecordOptions.ConfirmExit(GetVideoHostWindow(),&RecordManager);
}


bool CMainWindow::ConfirmChannelChange()
{
	if (!CmdLineParser.m_fTvRock)
		return true;
	return ShowMessage(TEXT("TvRockDTV^[QbgƂē쒆łB\n`lύX܂?"),TEXT("`lύX̊mF"),MB_OKCANCEL | MB_ICONQUESTION)==IDOK;
}


bool CMainWindow::CommandLineRecord(LPCTSTR pszFileName,DWORD Delay,DWORD Duration)
{
	CRecordManager::TimeSpecInfo StartTime,StopTime;

	if (Delay>0) {
		StartTime.Type=CRecordManager::TIME_DURATION;
		StartTime.Time.Duration=Delay*1000;
	} else {
		StartTime.Type=CRecordManager::TIME_NOTSPECIFIED;
	}
	if (Duration>0) {
		StopTime.Type=CRecordManager::TIME_DURATION;
		StopTime.Time.Duration=Duration*1000;
	} else {
		StopTime.Type=CRecordManager::TIME_NOTSPECIFIED;
	}
	return AppMain.StartRecord(pszFileName[0]!='\0'?pszFileName:NULL,
								&StartTime,&StopTime);
}


static bool SetCommandLineChannel(const CCommandLineParser *pCmdLine)
{
	const CChannelList *pChannelList;
	int i,j;

	for (i=0;(pChannelList=ChannelManager.GetChannelList(i))!=NULL;i++) {
		if (pCmdLine->m_TuningSpace>=0 && pCmdLine->m_TuningSpace!=i)
			continue;
		for (j=0;j<pChannelList->NumChannels();j++) {
			const CChannelInfo *pChannelInfo=pChannelList->GetChannelInfo(j);

			if ((pCmdLine->m_Channel==0
					|| pCmdLine->m_Channel==pChannelInfo->GetChannel())
				&& (pCmdLine->m_ControllerChannel==0
					|| pCmdLine->m_ControllerChannel==pChannelInfo->GetChannelNo())
				&& (pCmdLine->m_ServiceID==0
					|| pCmdLine->m_ServiceID==pChannelInfo->GetServiceID())
				&& (pCmdLine->m_NetworkID==0
					|| pCmdLine->m_NetworkID==pChannelInfo->GetNetworkID())
				&& (pCmdLine->m_TransportStreamID==0
					|| pCmdLine->m_TransportStreamID==pChannelInfo->GetTransportStreamID())) {
				return AppMain.SetChannel(i,j);
			}
		}
	}
	return false;
}


#ifndef _DEBUG
LONG WINAPI ExceptionFilter(EXCEPTION_POINTERS *ExceptionInfo)
{
	if (CmdLineParser.m_fSilent)
		return EXCEPTION_EXECUTE_HANDLER;
	return EXCEPTION_CONTINUE_SEARCH;
}
#endif


int APIENTRY _tWinMain(HINSTANCE hInstance,HINSTANCE /*hPrevInstance*/,
												LPTSTR pszCmdLine,int nCmdShow)
{
#ifdef _DEBUG
	_CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF/* | _CRTDBG_CHECK_ALWAYS_DF*/);
#else
	SetUnhandledExceptionFilter(ExceptionFilter);
#endif

	Logger.AddLog(TEXT("******** N ********"));

	hInst=hInstance;

	CoInitialize(NULL);

	if (pszCmdLine[0]!='\0')
		CmdLineParser.Parse(pszCmdLine);

	AppMain.Initialize();

	{
		TCHAR szDirectory[MAX_PATH];

		AppMain.GetAppDirectory(szDirectory);
		DriverManager.Find(szDirectory);
	}

	ColorSchemeOptions.SetApplyCallback(ColorSchemeApplyProc);
	ColorSchemeOptions.ApplyColorScheme();

	CMainWindow::Initialize();
	CStatusView::Initialize(hInst);

	{
		INITCOMMONCONTROLSEX iccex;

		iccex.dwSize=sizeof(INITCOMMONCONTROLSEX);
		iccex.dwICC=ICC_UPDOWN_CLASS | ICC_BAR_CLASSES | ICC_LISTVIEW_CLASSES | ICC_DATE_CLASSES | ICC_PROGRESS_CLASS;
		InitCommonControlsEx(&iccex);
	}

	if (!MainWindow.Create(NULL,WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
						   fNoTaskBar?WS_EX_TOOLWINDOW:0)) {
		MessageBox(NULL,TEXT("EBhE쐬ł܂B"),NULL,
														MB_OK | MB_ICONSTOP);
		return FALSE;
	}
	// ŃX^CςȂƃVXej[oȂ
	MainWindow.SetStyle(MainWindow.GetStyle()^WS_CAPTION,true);
	{
		RECT rc;

		MainWindow.GetClientRect(&rc);
		rc.bottom=StatusView.GetHeight();
		AdjustWindowRectEx(&rc,MainWindow.GetStyle(),FALSE,MainWindow.GetExStyle());
		MapWindowPoints(MainWindow.GetHandle(),NULL,(LPPOINT)&rc,2);
		MainWindow.SetPosition(&rc);
	}
	if (nCmdShow==SW_MINIMIZE)
		CmdLineParser.m_fMinimize=true;
	if (CmdLineParser.m_fStandby && CmdLineParser.m_fMinimize)
		CmdLineParser.m_fMinimize=false;
	if (!CmdLineParser.m_fStandby && !CmdLineParser.m_fMinimize) {
		MainWindow.AdjustPos();
		MainWindow.Show(/*nCmdShow*/SW_SHOWNOACTIVATE);
		MainWindow.Update();
	} else {
		MainWindow.Show(SW_HIDE);
	}

	ResidentManager.Initialize(MainWindow.GetHandle());
	if (CmdLineParser.m_fMinimize)
		MainWindow.InitMinimize();

	CoreEngine.SetDriverFileName(CmdLineParser.m_szDriverName[0]!='\0'?
								CmdLineParser.m_szDriverName:szDriverFileName);
	CoreEngine.m_DtvEngine.m_BonSrcDecoder.SetTracer(&Logger);
	if (!CmdLineParser.m_fNoDriver && !CmdLineParser.m_fStandby) {
		if (CoreEngine.GetDriverFileName()[0]!='\0') {
			StatusView.SetSingleText(TEXT("hCo̓ǂݍݒ..."));
			if (CoreEngine.LoadDriver()) {
				Logger.AddLog(TEXT("%s ǂݍ݂܂B"),CoreEngine.GetDriverFileName());
			} else {
				TCHAR szMessage[MAX_PATH+64];

				wsprintf(szMessage,TEXT("\"%s\" ǂݍ߂܂B"),CoreEngine.GetDriverFileName());
				Logger.AddLog(szMessage);
				MainWindow.PopupErrorMessage(szMessage);
			}
		} else {
			Logger.AddLog(TEXT("BonDriverݒ肳Ă܂B"));
			MainWindow.PopupMessage(
				TEXT("ݒŎgphCow肵ĂB"),
				TEXT("BonDriverݒ肳Ă܂B"),
				CMainWindow::MESSAGE_ICON_INFO);
		}
	}
	CoreEngine.SetDescramble(!CmdLineParser.m_fNoDescramble);
	CoreEngine.m_DtvEngine.SetDescrambleCurServiceOnly(fDescrambleCurServiceOnly);
	CoreEngine.m_DtvEngine.m_TsDescrambler.EnableSSE2(fDescrambleUseSSE2);
	CoreEngine.m_DtvEngine.m_TsDescrambler.EnableEmmProcess(fEnableEmmProcess);
	CoreEngine.m_DtvEngine.SetTracer(&StatusView);
	if (!CoreEngine.BuildDtvEngine(&DtvEngineHandler)) {
		/*
		TCHAR szText[1024];

		CoreEngine.FormatLastErrorText(szText,lengthof(szText));
		MessageBox(MainWindow.GetHandle(),szText,NULL,MB_OK | MB_ICONEXCLAMATION);
		*/
		Logger.AddLog(TEXT("DtvEngine\zł܂B"));
		MainWindow.PopupErrorMessage(TEXT("DtvEngine\zł܂B"));
	} else {
		if (!CmdLineParser.m_fStandby && !CoreEngine.OpenBcasCard()) {
			Logger.AddLog(TEXT("J[h[_I[vł܂B"));
			MainWindow.PopupErrorMessage(CoreEngine.GetLastErrorText());
		}
		if (CoreEngine.IsBcasCardOpen()) {
			LPCTSTR pszName=CoreEngine.m_DtvEngine.m_TsDescrambler.GetCardReaderName();

			if (pszName!=NULL) {
				TCHAR szCardID[32];

				CoreEngine.m_DtvEngine.m_TsDescrambler.FormatBcasCardID(szCardID,lengthof(szCardID));
				Logger.AddLog(TEXT("J[h[_ \"%s\" I[v܂"),pszName);
				Logger.AddLog(TEXT("(B-CASJ[hID %s / [J %c / o[W %d)"),
					szCardID,
					CoreEngine.m_DtvEngine.m_TsDescrambler.GetBcasCardManufacturerID(),
					CoreEngine.m_DtvEngine.m_TsDescrambler.GetBcasCardVersion());
			}
		}
		if (CoreEngine.IsDriverLoaded() && !CoreEngine.OpenDriver()) {
			Logger.AddLog(CoreEngine.GetLastErrorText());
			CoreEngine.UnloadDriver();
			if (!CmdLineParser.m_fSilent) {
				TCHAR szText[1024];

				CoreEngine.FormatLastErrorText(szText,lengthof(szText),
									TEXT("BonDriveȑł܂B\n"));
				MainWindow.PopupMessage(szText);
			}
		}
	}

	StatusView.SetSingleText(TEXT("`lݒǂݍł܂..."));
	AppMain.InitializeChannel();

	if (!CmdLineParser.m_fNoPlugin) {
		TCHAR szPluginDir[MAX_PATH];

		StatusView.SetSingleText(TEXT("vOCǂݍł܂..."));
		AppMain.GetAppDirectory(szPluginDir);
		::PathAppend(szPluginDir,TEXT("RecPlugins"));
		PluginList.LoadPlugins(szPluginDir);
		PluginOptions.RestorePluginOptions();
	}

	CoreEngine.m_DtvEngine.SetTracer(NULL);
	StatusView.SetSingleText(NULL);

	if (CoreEngine.m_DtvEngine.IsSrcFilterOpen()) {
		if (CoreEngine.IsUDPDriver()) {
			if (CmdLineParser.m_NetworkDriverPort>1234
					&& CmdLineParser.m_NetworkDriverPort<=1243)
				MainWindow.PostCommand(CM_CHANNEL_FIRST+(CmdLineParser.m_NetworkDriverPort-1234));
			if (CmdLineParser.m_ControllerChannel>0)
				SetCommandLineChannel(&CmdLineParser);
		} else if (CmdLineParser.IsChannelSpecified()) {
			SetCommandLineChannel(&CmdLineParser);
		} else if (RestoreChannelInfo.Space>=0 && RestoreChannelInfo.Channel>=0
				/*&& lstrcmpi(RestoreChannelInfo.szDriverName,CoreEngine.GetDriverFileName())==0*/) {
			const CChannelList *pList=ChannelManager.GetCurrentChannelList();
			int i;

			i=pList->Find(RestoreChannelInfo.Space,RestoreChannelInfo.Channel,
						  RestoreChannelInfo.ServiceID);
			if (i>=0) {
				MainWindow.PostCommand(CM_CHANNEL_FIRST+i);
			} else {
				pList=ChannelManager.GetChannelList(RestoreChannelInfo.Space);
				if (pList!=NULL) {
					i=pList->Find(RestoreChannelInfo.Space,
								  RestoreChannelInfo.Channel,
								  RestoreChannelInfo.ServiceID);
					if (i<0 && RestoreChannelInfo.ServiceID>0)
						i=pList->Find(RestoreChannelInfo.Space,
									  RestoreChannelInfo.Channel);
					if (i>=0)
						AppMain.SetChannel(RestoreChannelInfo.Space,i);
				}
			}
		} else {
			// `lɐݒ肷
			const CChannelList *pList=ChannelManager.GetCurrentChannelList();
			int i=pList->Find(
				CoreEngine.m_DtvEngine.m_BonSrcDecoder.GetCurSpace(),
				CoreEngine.m_DtvEngine.m_BonSrcDecoder.GetCurChannel(),0);

			if (i>=0)
				MainWindow.PostCommand(CM_CHANNEL_FIRST+i);
		}

		if (CmdLineParser.m_fRecord)
			MainWindow.CommandLineRecord(CmdLineParser.m_szRecordFileName,
				CmdLineParser.m_RecordDelay,CmdLineParser.m_RecordDuration);
	}
	if (CmdLineParser.m_fStandby)
		MainWindow.InitStandby();
	if (CmdLineParser.m_fExitOnRecordEnd)
		MainWindow.SendCommand(CM_EXITONRECORDINGSTOP);

	if (CmdLineParser.m_fTvRock) {
		TvRockDTVTarget.SetTracer(&Logger);
		if (CmdLineParser.m_szTerrestrialSpaceName[0]!='\0')
			TvRockDTVTarget.SetTerrestrialSpace(CmdLineParser.m_szTerrestrialSpaceName);
		if (TvRockDTVTarget.Initialize()) {
			RECT rc;

			MainWindow.GetClientRect(&rc);
			rc.right=CAPTION_WIDTH;
			::InvalidateRect(MainWindow.GetHandle(),&rc,TRUE);
			ResidentManager.SetDeviceID(TvRockDTVTarget.GetDeviceID());
		} else {
			Logger.AddLog(TvRockDTVTarget.GetLastErrorText());
			if (!CmdLineParser.m_fSilent) {
				TCHAR szText[256];

				TvRockDTVTarget.FormatLastErrorText(szText,lengthof(szText));
				MainWindow.PopupMessage(szText);
			}
		}
	}
	if (CmdLineParser.m_fUdp || CmdLineParser.m_fTcp)
		MainWindow.PostCommand(CM_UDPSEND);

	MSG msg;

	while (GetMessage(&msg,NULL,0,0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	CoUninitialize();

	Logger.AddLog(TEXT("******** I ********"));
	if (CmdLineParser.m_fSaveLog && !Logger.GetOutputToFile()) {
		TCHAR szFileName[MAX_PATH];

		Logger.GetDefaultLogFileName(szFileName);
		Logger.SaveToFile(szFileName,true);
	}

	return (int)msg.wParam;
}
